#Contributing

Here's the most direct way to get your work merged into the project:

If your work is a new feature or a patch then follow the following workflow

- Clone the project ( `git clone git@github.com:DigitalInnovation/mns-core-ui-header.git` ).
- Create a feature branch to contain your change ( `git checkout -b my_awesome_feature` ).
- Code away and add tests if applicable. Not necessarily in that order.
- Make sure everything still passes by running `npm run test-unit`.
- Update the change log with details of the change under the corresponding heading (new feature/patch) - create the heading if it doesn't already exist. The version of the next published version will then be decided by this.
- If necessary, rebase your commits into logical chunks, without errors.
- Push the branch up ( `git push origin my_awesome_feature` ).
- Create a pull request against mns-core-ui-header/master and describe what your change does and the why you think it should be merged.

#Coding guidelines

Click [here]((https://github.com/DigitalInnovation/mns-core-ui/blob/master/src/readme.md)) to see what coding guidelines (naming conventions, testing standards) we follow. 

#Set up

##Prerequisites
- [Node](https://nodejs.org/en/) / npm (see [package.json](https://github.com/DigitalInnovation/mns-core-ui/blob/master/package.json) engine properties)

##Installation
- Run `npm install`
- Run `npm start`
- See the app running at `http://localhost:3000/`

##Features

- Express web server
- JS/SASS linting
- Server side handlebars templating engine
- ES6 transpile support
- Minification and sourcemap support
- Unit testing framework (Istanbul coverage, mocha, chai)
- Distribution and Development builds

##Development environment
To start the app in dev mode, run `npm run dev`

- Live reloading on file changes (JS/SASS)
    - JS file change executes JS linting and unit tests
    - SASS change executes sass linting
- All assets are rebundled on JS/SASS changes


##Bundling assets
Run `npm run build:assets` to compile and bundle all assets to dist folder.


##Unit testing

Run `npm run test:unit` to run all unit tests in `src` directory.

Overall test coverage and file wise coverage will be displayed in the console.

To view detailed report, open `/coverage/lcov-report/index.html` file in a browser.
This will give detailed view of test coverage. You will be able to see code coverage (in %) for all folders subfolders and files. Also you will get a visual report of functions and lines covered or not covered.

If user is running application in dev environment using `npm run dev` command, unit tests will also run once. Once the server is up and running, any change in any `js` file in `src` folder will run the unit test suit again.

##JS Linting

Run `npm run lint:js` to lint all JS files in the src/ directory and webpack config files

##SASS Linting

Run `npm run lint:sass` to check linting issues in all `.scss` and `.sass` files in `src` directory.

A config file named `.sass-lint.yml` is added at the root folder which contains the list of files included for lint test and rules. This overrides the default config file which is available with same name in node module.

If user is running application in dev environment using `npm run dev` command, sass lint check will also run once.

## CI

### Setup and login:

fly -t dachs login --concourse-url https://ci.dachs.dog --team-name=boilerteam

This will save a new target locally, you can then use the following command for all subsequent logins

fly -t dachs login -n boilerteam

Copy and paste the generated url into your browser:

https://ci.dachs.dog/auth/github?team_name=boilerteam

This will return an associated bearer token. Enter the bearer token to login.


### Deploy new configuration:

To deploy new configuration to the server:

1. Copy env_template.yml to env.yml (env.yml is already added to the git ignore)
2. Fill in credentials and other configuration specific to your project
3. Deploy configuration to the server using --load-vars-from ci/env.yml
4. Do not commit the env.yml file

PR Pipeline -

  fly -t dachs set-pipeline -p pr-pipeline -c ci/pipelines/pr_pipline.yml --load-vars-from ci/env.yml

Build Test Deploy Pipeline -

  fly -t dachs set-pipeline -p dev-pipeline -c ci/pipelines/dev_pipline.yml --load-vars-from ci/env.yml
