var path = require('path');
var entryPoint;
var sassPaths = [];
var mnsCoreUi = require('mns-core-ui');

function getLibrarySourcePath() {
    return path.join('src', 'mns-core-ui-header', 'templates', 'header');
}

function getConfigDataPath() {
    return path.join(entryPoint, getLibrarySourcePath(), 'data', 'config');
}

/**
 * will define entry point to module depending
 * on if the module is installed in an application
 * or is running in development mode.
 */
try {
    entryPoint = path.dirname(require.resolve('mns-core-ui-header'));
} catch (e) {
    entryPoint = process.cwd();
}

var ExposeRoutes = require(path.join(entryPoint, getLibrarySourcePath(), 'mocks', 'ExposeRoutes'));
var router = require(path.join(entryPoint, getLibrarySourcePath(), 'mocks', 'routes'));

sassPaths.push(path.join(entryPoint, 'src'));
sassPaths.push(path.join(entryPoint, 'dist'));
sassPaths = sassPaths.concat(mnsCoreUi.sassPaths);

var assetsDir = path.join(entryPoint, '/dist/templates/header/assets');

function getConfig(storeIdentifier) {
    if (storeIdentifier === undefined)
    {
        storeIdentifier = 'marksandspencer';
    }
    var headerConfig = require(path.join(getConfigDataPath(), storeIdentifier, 'header.js'));
    var CONFIG_KEY = Object.keys(headerConfig)[0];
    headerConfig[CONFIG_KEY].loyaltyApiHost = `${process.env.LOYALTY_API_HOST}/loyalty-service/`;
    var config = {
        key: CONFIG_KEY,
        data: headerConfig,
        decorators: {
            labels: require(path.join(getConfigDataPath(), storeIdentifier, 'labels.js')).labels,
            featureSwitch: require(path.join(getConfigDataPath(), storeIdentifier, 'featureSwitch.js')).featureSwitch,
            features: require(path.join(getConfigDataPath(), storeIdentifier, 'featureSwitch.js')).features
        }
    };
    return config;
}

module.exports = {
    ExposeRoutes: ExposeRoutes,
    router: router,
    config: getConfig,
    sassPaths: sassPaths,
    assetPaths: assetsDir,
    templatePaths: {
        html: path.join(entryPoint, 'dist', 'templates')
    }
};
