'use strict';
define([
    '../data/config/app',
    '../../../lib/domQ',
    './Search',
    './util',
    './DropdownMenu',
    './miniBag/Index',
    './navigation/Breakpoint',
    './links/Index',
    './savedItems/SavedItems',
    './links/cookieService',
    './loyalty/Index',
    './analytics',
    './MyBag',
    './HdrSiteStripe',
    './stickyHeader',
    './cookieBanner.js'

], function (config, domQ, Search, util, DropdownMenu, MiniBagOverlay, Breakpoint, HeaderLinks, SavedItems, mnsCookieService, LoyaltyDropdown, analytics, MyBag, HdrSiteStripe, stickyHeader, cookieBanner) {

    function Header() {

        var requestParams = {
            storeId: config.storeId,
            langId: config.langId,
            catalogId: config.catalogId
        };
        var CLEAR = 'Clear',
            CLR_RECENT_SEARCHES_MD = 'Clear recent searches';
        // toggle header links
        new HeaderLinks(document.querySelector('#progressiveHeaderSection')).show();

        // bag
        new MyBag({
            myBagCount: document.querySelector('.navigation-nav-header__bag-count'),
            myBagElement: document.getElementsByClassName('navigation-res-nav-header__item--bag')[0],
            checkoutElement: document.getElementsByClassName('navigation-res-nav-header__item--checkout')[0],
            myBagIconElement: document.querySelector('.navigation-res-nav-header__item--bag .navigation-primary__menu-icon')
        });
        //Cookie Banner
        if (!document.cookie.match('CONSENTMGR')) {
            cookieBanner.bind();
        }
        // Load syte CDN after page load event
        if (window.addEventListener) {
            window.addEventListener('load', enableSyteCdn, false);
        }
        document.addEventListener('touchend', touchHandlerForDropdown, true);

        /* my account bindings */
        var myAccountSparkLink = document.querySelector('.navigation-res-nav-header__item.navigation-myaccount  #myAccountSparkLink');
        var signInSparkLink = document.querySelector('.navigation-header__sparks-link-signin');
        var sparksUserLink = 'data-sparks-login-link';
        if (mnsCookieService.isUserLoggedIn() && mnsCookieService.isUserSparksMember()) {
            sparksUserLink = 'data-sparks-hub-link';
        }

        if (myAccountSparkLink) {
            if (myAccountSparkLink.getAttribute(sparksUserLink).indexOf('intid') !== -1) {
                var sparksUrl = myAccountSparkLink.getAttribute(sparksUserLink).replace(/(intid=).*?(#|$)/, '$1' + 'header_account_sparks');
                myAccountSparkLink.setAttribute(sparksUserLink, sparksUrl);
                myAccountSparkLink.setAttribute('href', myAccountSparkLink.getAttribute(sparksUserLink));
            } else {
                var sparksurl = myAccountSparkLink.getAttribute(sparksUserLink);
                var newSparksurl = sparksurl + '#intid=header_account_sparks';
                myAccountSparkLink.setAttribute(sparksUserLink, newSparksurl);
                myAccountSparkLink.setAttribute('href', myAccountSparkLink.getAttribute(sparksUserLink));
            }
        }

        if (signInSparkLink) {
            if (signInSparkLink.getAttribute(sparksUserLink).indexOf('intid') !== -1) {
                var sparksUrl1 = signInSparkLink.getAttribute(sparksUserLink).replace(/(intid=).*?(#|$)/, '$1' + 'header_sign-in_sparks');
                signInSparkLink.setAttribute(sparksUserLink, sparksUrl1);
                signInSparkLink.setAttribute('href', signInSparkLink.getAttribute(sparksUserLink));
            } else {
                var sparksurl1 = signInSparkLink.getAttribute(sparksUserLink);
                var newSparksurl1 = sparksurl1 + '#intid=header_sign-in_sparks';
                signInSparkLink.setAttribute(sparksUserLink, newSparksurl1);
                signInSparkLink.setAttribute('href', signInSparkLink.getAttribute(sparksUserLink));
            }

        }

        // add toggle behavior to all #menuBarTop dropdowns (remove 'find a store' explicitly, 'search' & 'save for later' not included)
        var dropdownMenu = new DropdownMenu(document.getElementById('menuBarTop'), {
            menuItem: '.navigation-dropdown:not(.navigation-res-nav-header__item--location)',
            menuItemLink: '.navigation-dropdown__menuitem-link',
            menuDropdown: '.navigation-dropdown__panel-wrapper',
            namespace: 'HEADER_DROPDOWN_MENU'
        });

        // Search typeahead
        var searchContainer = document.querySelector('.navigation-res-nav-header__dropdown--search'),
            searchCommand = searchContainer.querySelector('#searchSuggestionCmd').value,
            navVeil = document.querySelector('.navigation-veil-search'),
            searchContainerParentOverlay = document.querySelector('.navigation-search-overlay'),
            searchButton = document.querySelector('.navigation-btn__icon--search');

        var search = new Search({
            'searchContainer': searchContainer,
            'command': config.wcsBaseUrl + searchCommand,
            'navVeil': navVeil,
            'searchContainerParentOverlay': searchContainerParentOverlay,
            'searchButton': searchButton
        });
        search.init(requestParams.storeId);

        // add toggle behavior to 'search' & 'save for later' dropdowns for xsmall/small viewports
        this.searchIconDisplay = document.querySelector('.navigation-dropdown__panel--search');
        this.headerSection = document.getElementById('progressiveHeaderSection');
        if (Breakpoint.size() === 'xsmall') {
            //dropdown__panel-wrapper
            if (this.searchIconDisplay) {
                domQ(this.searchIconDisplay).removeClass('navigation-dropdown__panel-wrapper', 'navigation-dropdown__panel--search');
                domQ(this.searchIconDisplay).addClass('navigation-dropdown__panel-wrapper__new', 'navigation-dropdown__panel--search');
                dropdownMenu.addMenuItemLink(document.querySelector('.navigation-btn__icon--search-container.navigation-dropdown__link'), document.querySelector('.navigation-dropdown__panel-wrapper__new.navigation-dropdown__panel--search'));
                dropdownMenu.addMenuItemLink(document.querySelector('.res-nav-header__fav-icon.navigation-dropdown__menuitem-link'), document.querySelector('.navigation-res-nav-header__item--fav .navigation-dropdown__panel-wrapper'));
            }
            if (!window.mobileSearchKey) { // collapsed search view
                domQ(this.headerSection).addClass('navigation-header-height');
            } else { // exposed search view
                domQ(this.headerSection).removeClass('navigation-header-height');
            }
        } else if (Breakpoint.size() === 'small') {
            if (this.searchIconDisplay) {
                domQ(this.searchIconDisplay).removeClass('navigation-dropdown__panel-wrapper__new', 'navigation-dropdown__panel--search');
                domQ(this.searchIconDisplay).addClass('navigation-dropdown__panel-wrapper', 'navigation-dropdown__panel--search');
                dropdownMenu.addMenuItemLink(document.querySelector('.navigation-btn__icon--search-container.navigation-dropdown__link'), document.querySelector('.navigation-dropdown__panel-wrapper.navigation-dropdown__panel--search'));
            }
        }
        // add toggle behavior to 'find a store' dropdown for other than xsmall viewport
        var findStoreItem = document.querySelector('.navigation-res-nav-header__item--location'),
            findStoreIcon = findStoreItem ? findStoreItem.querySelector('.navigation-res-nav-header__icon--location') : null;
        // Adding store location url parameters
        if (findStoreIcon) {
            findStoreIcon.href += '?langId=' + requestParams.langId + '&storeId=' + requestParams.storeId + '&catalogId=' + requestParams.catalogId;
        }

        // add url params to 'find store' button in gnav for xsmall viewport
        var findStoreItemBottom = document.querySelector('.navigation-bottom--find-store'),
            findStoreIconBottom = findStoreItemBottom ? findStoreItemBottom.querySelector('.nav-primary__menu-link--find-store') : null;
        // Adding store location url parameters
        if (findStoreIconBottom) {
            findStoreIconBottom.href += '?langId=' + requestParams.langId + '&storeId=' + requestParams.storeId + '&catalogId=' + requestParams.catalogId;
        }

        // initialize mini bag overlay for iPad/large viewport
        if (Breakpoint.size() === 'small') {
            MiniBagOverlay.initialize(document.querySelectorAll('#miniBagOverlayHeader'), {
                miniBagCMD: config.wcsBaseUrl + config.CMD.miniBasket + '?langId=' + requestParams.langId + '&storeId=' + requestParams.storeId + '&catalogId=' + requestParams.catalogId
            });
        }
        if (Breakpoint.size() === 'medium' || Breakpoint.size() === 'large') {
            MiniBagOverlay.initialize(document.querySelectorAll('#miniBagOverlayHeader'), {
                miniBagCMD: config.wcsBaseUrl + config.CMD.miniBasket + '?langId=' + requestParams.langId + '&storeId=' + requestParams.storeId + '&catalogId=' + requestParams.catalogId
            });
            this.dummySearchPanel = document.querySelector('.navigation-dropdown__panel--search');
            this.dummySearchPanel.addEventListener('click', function () {
                this.crosselement = document.querySelector('.navigation-dock-search-popup .navigation-search-bar--clear');
                if (this.crosselement) {
                    this.crosselement.addEventListener('keydown', crossElementBlur);
                }
            });
            this.dummySearchPanel.addEventListener('keyup', function (event) {
                var keyPressed = event.key || event.keyCode;
                if ((keyPressed === 'Enter' || keyPressed === 13)) {
                    this.crosselement = document.querySelector('.navigation-dock-search-popup .navigation-search-bar--clear');
                    if (this.crosselement) {
                        this.crosselement.addEventListener('keydown', crossElementBlur);
                    }
                }
            });
        }

        function crossElementBlur(event) {
            this.SuggessionListItem = document.querySelectorAll('.navigation-search-item--list a');
            this.clearResetSearchLink = document.querySelector('.navigation-suggestion-item--clear-text');
            this.lastSuggessionListItem = this.SuggessionListItem[this.SuggessionListItem.length - 1];
            if (!this.clearResetSearchLink && !this.lastSuggessionListItem && !window.searchDropDownClicked) {
                closeSearchOverlayOnTab(event);
            } else {
                if (this.clearResetSearchLink) {
                    this.clearResetSearchLink.addEventListener('keydown', closeSearchOverlayOnTab);
                } else if (this.lastSuggessionListItem) {
                    this.lastSuggessionListItem.addEventListener('keydown', closeSearchOverlayOnTab);
                }
            }
            window.searchDropDownClicked = false;
        }

        function closeSearchOverlayOnTab(event) {
            var keyPressed = event.key || event.keyCode;
            if (keyPressed === 'Tab' || keyPressed === 9) {
                if (window.clearSearch) {
                    window.clearSearch = false;
                    return false;
                }
                if (document.querySelector('.navigation-dock-search-popup .navigation-search-bar--clear') !== event.target) {
                    event.target.removeEventListener('keydown', closeSearchOverlayOnTab);
                }
                window.l1focus = true;
                search.resetSearchText();
                search.formParams.q = '';
                search.resetSuggestions(event);
                search.closeSuggestions();
                search.hidePopup();
                var dummySearchPanel = document.querySelector('.navigation-dropdown__panel--search');
                dummySearchPanel.focus();
                event.preventDefault();
            }
        }
        if (document.getElementById('sparksDropdown')) {
            var loyaltyOfferApi = document.getElementById('sparksDropdown').getAttribute('data-loyalty-api-host');

            var loyaltyDropdown = new LoyaltyDropdown(document.querySelector('header .navigation-res-nav-header__item--sparks'), {
                authTokenUrl: config.wcsBaseUrl + config.CMD.authToken,
                loyaltyOfferApi: loyaltyOfferApi + 'api/offers'
            });


            // set loyalty dropdown
            if (mnsCookieService.isUserLoggedIn() && mnsCookieService.isUserSparksMember()) {
                loyaltyDropdown.initialize();
            }

            //toggle link for sparks in sign-in drop-down
            var elDesktop = document.querySelector('.navigation-header__sparks-link-signin');
            if (elDesktop) {
                loyaltyDropdown.toggleSparksLinkUrl(elDesktop);
            }

            //toggle link for sparks in sign-in drop-down- mobile
            var elMobile = document.querySelector('.navigation-header__sparks-link-signin-mobile');
            if (elMobile) {
                var url = mnsCookieService.isUserSparksMember() ? elMobile.getAttribute('data-sparks-hub-link') : elMobile.getAttribute('data-sparks-login-link');
                elMobile.setAttribute('data-href', url);
            }
        }

        // set tabindex for visually hidden elements
        var notVisibleToAllBreakpoints = '.navigation-dropdown__panel-wrapper a, .navigation-dropdown__panel-wrapper button[type=reset], .navigation-primary__section>ul>li>a, .navigation-suggestion-item--recent';
        var selectorsToRemoveMdToLrgBreakpoint = '.navigation-res-nav-header__icon--fav, .navigation-btn__icon--search-container';
        var selectorsToRemoveSmallBreakpoints = '.navigation-dropdown__panel-wrapper input[type=search]';
        var extraSelectors = (Breakpoint.size() === 'medium' || Breakpoint.size() === 'large') ? selectorsToRemoveMdToLrgBreakpoint : selectorsToRemoveSmallBreakpoints;
        var selectorsToRemove = document.querySelectorAll(notVisibleToAllBreakpoints + extraSelectors);
        var navList = document.querySelector('.navigation-primary__section');
        var navL23 = document.getElementById('navContainer');
        util.setTabIndex(selectorsToRemove, -1);

        HdrSiteStripe.initHdrSiteStripeCarousel(requestParams.storeId);
        var _this = this;
        // when tablet orientation changes
        window.addEventListener('resize', onResizeEvent);

        function onResizeEvent() {
            var promoSlides = document.querySelectorAll('.navigation-sitestripe__slide');
            var isIEorEdge = util.checkIeVersion();
            var ipadMenu = document.querySelector('.navigation-hamburger-trigger');

            for (var index = 0; index < promoSlides.length; index++) {
                promoSlides[index].removeAttribute('style');
                if (!isIEorEdge) {
                    promoSlides[index].firstElementChild.removeAttribute('style');
                }
            }
            // bind the 'search' dropdown
            if (Breakpoint.size() === 'xsmall') {
                var searchInput = document.querySelector('.navigation-search-overlay .navigation-res-nav-header__dropdown--search .navigation-search-bar--search');
                searchInput.autofocus = true;
                navList.style.transition = 'none';
                _this.searchIconDisplay = document.querySelector('.navigation-dropdown__panel--search');
                _this.searchButton = document.querySelector('.navigation-res-nav-header__search .navigation-search-bar--submit');
                _this.headerSection = document.getElementById('progressiveHeaderSection');
                document.querySelector('.navigation-btn__icon--search').innerText = '';
                document.querySelector('.navigation-header__title-text__desktop').innerText = '';
                if (_this.searchIconDisplay) {
                    domQ(_this.searchIconDisplay).removeClass('navigation-dropdown__panel-wrapper', 'navigation-dropdown__panel--search');
                    domQ(_this.searchIconDisplay).addClass('navigation-dropdown__panel-wrapper__new', 'navigation-dropdown__panel--search');
                    dropdownMenu.addMenuItemLink(document.querySelector('.navigation-btn__icon--search-container.navigation-dropdown__link'), document.querySelector('.navigation-dropdown__panel-wrapper__new.navigation-dropdown__panel--search'));
                }
                if (!document.querySelector('.navigation-sitestripe').classList.contains('navigation-no-stripe') && !window.siteStripeKey) {
                    document.querySelector('.navigation-sitestripe').style.display = 'none';
                    document.querySelector('.navigation-res-nav-header__item--menu').classList.remove('navigation-stripe');
                }
                var userAgent = window.navigator.userAgent || window.navigator.vendor || window.opera;
                var dummyVisualSearch = document.querySelector('.navigation-res-nav-header__search .navigation-visual-search');
                var dummySearchBar = document.querySelector('.navigation-res-nav-header__search .navigation-search-bar--search-wrap');
                if (/iPad/.test(userAgent) && (window.innerWidth === 768 || window.innerWidth === 767)) {
                    document.querySelector('.navigation-res-nav-header__search').style.display = 'block';
                    document.querySelector('.navigation-btn__icon--search').style.display = 'none';
                    document.querySelector('.navigation-btn__icon--search-container').style.display = 'none';
                    dummyVisualSearch = document.querySelector('.navigation-res-nav-header__search .navigation-visual-search');
                    domQ(_this.headerSection).hasClass('navigation-header-height') ? domQ(_this.headerSection).removeClass('navigation-header-height') : '';
                    if (dummyVisualSearch) {
                        dummyVisualSearch.style.display = 'block';
                    } else {
                        dummySearchBar = document.querySelector('.navigation-res-nav-header__search .navigation-search-bar--search-wrap');
                        domQ(dummySearchBar).addClass('navigation-search-bar--search-wrap__full-width');
                    }
                } else if (window.mobileSearchKey) { //exposed on resize
                    document.querySelector('.navigation-res-nav-header__search').style.display = 'block';
                    document.querySelector('.navigation-btn__icon--search').style.display = 'none';
                    document.querySelector('.navigation-btn__icon--search-container').style.display = 'none';
                    dummyVisualSearch = document.querySelector('.navigation-res-nav-header__search .navigation-visual-search');
                    domQ(_this.headerSection).hasClass('navigation-header-height') ? domQ(_this.headerSection).removeClass('navigation-header-height') : '';
                    if (dummyVisualSearch) {
                        dummyVisualSearch.style.display = 'block';
                    } else {
                        dummySearchBar = document.querySelector('.navigation-res-nav-header__search .navigation-search-bar--search-wrap');
                        domQ(dummySearchBar).addClass('navigation-search-bar--search-wrap__full-width');
                    }
                } else { //collapsed on resize
                    document.querySelector('.navigation-res-nav-header__search').style.display = 'none';
                    document.querySelector('.navigation-btn__icon--search').style.display = 'block';
                    domQ(_this.headerSection).addClass('navigation-header-height');
                }
                if ((window.innerWidth !== 767) || (window.innerWidth !== 768)) {
                    _this.mnslogo = document.querySelector('.navigation-logo');
                    domQ(_this.mnslogo).hasClass('navigation-logo-ipad-portrait') ? domQ(_this.mnslogo).removeClass('navigation-logo-ipad-portrait') : '';
                    _this.ipadinput = document.querySelector('.navigation-dropdown__panel--search');
                    if (_this.ipadinput) {
                        domQ(_this.ipadinput).hasClass('navigation-search-bar-ipad-portrait') ? domQ(_this.ipadinput).removeClass('navigation-search-bar-ipad-portrait') : '';
                    }
                    _this.Ipadresize = document.querySelector('.navigation-res-nav-header__search .navigation-search-bar--search-wrap');
                    if (_this.Ipadresize) {
                        domQ(_this.Ipadresize).hasClass('navigation-search-warp-ipad-portrait') ? domQ(_this.Ipadresize).removeClass('navigation-search-warp-ipad-portrait') : '';
                    }
                    ipadMenu = document.querySelector('.navigation-hamburger-trigger');
                    if (ipadMenu) {
                        domQ(ipadMenu).hasClass('navigation-hamburger-ipad-portrait') ? domQ(ipadMenu).removeClass(ipadMenu, 'navigation-hamburger-ipad-portrait') : '';
                    }
                    if (_this.headerSection) {
                        domQ(_this.headerSection).hasClass('navigation-header-ipad-height') ? domQ(_this.headerSection).removeClass('navigation-header-ipad-height') : '';
                    }
                }
            } else {
                var body = document.querySelector('body');
                body.addEventListener('click', activeElementListener);
                body.addEventListener('keyup', firstTab);
                navList.removeAttribute('style');
                navL23.removeAttribute('style');
                _this.searchIconDisplay = document.querySelector('.navigation-dropdown__panel-wrapper__new');
                document.querySelector('.navigation-res-nav-header__search .navigation-search-bar--search').removeAttribute('placeholder');
                document.querySelector('.navigation-res-nav-header__search .navigation-search-bar--search').setAttribute('placeholder', 'Search');
                document.querySelector('.navigation-res-nav-header__search').style.display = 'block';
                document.querySelector('.navigation-header__title-text__desktop').innerText = '';
                document.querySelector('.navigation-btn__icon--search').style.display = 'block';
                domQ(_this.headerSection).hasClass('navigation-header-height') ? domQ(_this.headerSection).removeClass('navigation-header-height') : '';
                if (_this.searchIconDisplay) {
                    domQ(_this.searchIconDisplay).removeClass('navigation-dropdown__panel-wrapper__new', 'navigation-dropdown__panel--search');
                    domQ(_this.searchIconDisplay).addClass('navigation-dropdown__panel-wrapper', 'navigation-dropdown__panel--search');
                    dropdownMenu.addMenuItemLink(document.querySelector('.navigation-btn__icon--search-container.navigation-dropdown__link'), document.querySelector('.navigation-dropdown__panel-wrapper.navigation-dropdown__panel--search'));
                }
            }

            if (document.querySelector('.navigation-sitestripe').classList.contains('navigation-no-stripe')) {
                HdrSiteStripe.initHdrSiteStripeCarousel(requestParams.storeId);
            } else if (Breakpoint.size() !== 'xsmall') {
                document.querySelector('.navigation-sitestripe').style.display = 'block';
            }
            // Ipad portrait to land scape responsive code
            if (Breakpoint.size() !== 'xsmall') {
                _this.dockSearchPopup = document.querySelector('.navigation-dock-search-popup');
                _this.searchBarSuggestion = document.querySelector('.navigation-search-bar--suggestion-dropdown');
                _this.searchsuggestionList = document.querySelector('.navigation-search-item--list li');
                _this.searchbardropdownDiv = document.querySelector('.navigation-search-bar--dropdowndiv');
                _this.searchSuggestionTitle = document.querySelector('.navigation-suggestion-item--title');
                _this.clearRecentSearchText = document.querySelector('.navigation-suggestion-item--clear-text');
                _this.suggestionItemClear = document.querySelector('.navigation-suggestion-item--title .navigation-suggestion-item--clear');
                _this.searchClearRecent = document.querySelector('.navigation-recentsearch--list .navigation-suggestion-item--recent');
                if (_this.dockSearchPopup) {
                    domQ(_this.dockSearchPopup).hasClass('navigation-hdr-sticky-class') ? domQ(_this.dockSearchPopup).removeClass('navigation-hdr-sticky-class') : '';
                    if (_this.searchBarSuggestion) {
                        document.querySelector('.navigation-search-bar--suggestion-dropdown').removeEventListener('touchstart', search.stickySearch);
                    }
                }
                if (_this.searchBarSuggestion) {
                    domQ(_this.searchBarSuggestion).hasClass('navigation-suggestion-list-padding') ? domQ(_this.searchBarSuggestion).removeClass('navigation-suggestion-list-padding') : '';
                }
                if (_this.SearchsuggestionList) {
                    domQ(_this.searchbardropdownDiv).hasClass('navigation-scroll-height') ? domQ(_this.searchbardropdownDiv).removeClass('navigation-scroll-height') : '';
                }
                if (_this.suggestionItemClear) {
                    _this.suggestionItemClear.innerText = '';
                    domQ(_this.suggestionItemClear).hasClass('navigation-suggestion-item--clear') ? domQ(_this.suggestionItemClear).removeClass('navigation-suggestion-item--clear') : '';

                }
                if (_this.clearRecentSearchText && _this.searchClearRecent) {
                    _this.clearRecentSearchText.innerText = 'Clear';
                }
                if (_this.searchSuggestionTitle && !_this.clearRecentSearchText) {
                    _this.clearElement = document.createElement('div');
                    _this.recentElement = document.querySelector('.navigation-suggestion-item--title');
                    _this.recentSearchList = document.querySelector('.navigation-recentsearch--list');
                    _this.clearElementdevice = document.createElement('span');
                    _this.clearElementdevice.tabIndex = '-1';
                    _this.clearElementdevice.setAttribute('aria-label', 'Clear');
                    _this.clearElementdevice.innerHTML = 'Clear';
                    domQ(_this.clearElementdevice).addClass('navigation-suggestion-item--clear-text');
                    domQ(_this.clearElement).addClass('suggestion-item');
                    domQ(_this.clearElement).addClass('navigation-suggestion-item--clear');
                    _this.clearElement.appendChild(_this.clearElementdevice);
                    if (_this.recentSearchList) {
                        _this.recentSearchList.appendChild(_this.clearElement);
                    }
                    _this.clearElementdevice.addEventListener('click', clearElmntDevice);
                }

                //Ipad 767 view portrait to landscape
                if ((window.innerWidth !== 767) || (window.innerWidth !== 768)) {
                    _this.mnslogo = document.querySelector('.navigation-logo');
                    domQ(_this.mnslogo).hasClass('navigation-logo-ipad-portrait') ? domQ(_this.mnslogo).removeClass('navigation-logo-ipad-portrait') : '';
                    _this.ipadinput = document.querySelector('.navigation-dropdown__panel--search');
                    if (_this.ipadinput) {
                        domQ(_this.ipadinput).hasClass('navigation-search-bar-ipad-portrait') ? domQ(_this.ipadinput).removeClass('navigation-search-bar-ipad-portrait') : '';
                    }
                    _this.Ipadresize = document.querySelector('.navigation-res-nav-header__search .navigation-search-bar--search-wrap');
                    if (_this.Ipadresize) {
                        domQ(_this.Ipadresize).hasClass('navigation-search-warp-ipad-portrait') ? domQ(_this.Ipadresize).removeClass('navigation-search-warp-ipad-portrait') : '';
                    }
                    ipadMenu = document.querySelector('.navigation-hamburger-trigger');
                    if (ipadMenu) {
                        domQ(ipadMenu).hasClass('navigation-hamburger-ipad-portrait') ? domQ(ipadMenu).removeClass(ipadMenu, 'navigation-hamburger-ipad-portrait') : '';
                    }
                    if (_this.headerSection) {
                        domQ(_this.headerSection).hasClass('navigation-header-ipad-height') ? domQ(_this.headerSection).removeClass('navigation-header-ipad-height') : '';
                    }
                }
            } else {
                _this.searchBarSuggestion = document.querySelector('.navigation-search-bar--suggestion-dropdown');
                _this.searchCleartext = document.querySelector('.navigation-suggestion-item--title .suggestion-item');
                _this.clearRecentSearchText = document.querySelector('.navigation-suggestion-item--clear-text');
                if (_this.searchBarSuggestion) {
                    document.querySelector('.navigation-search-bar--suggestion-dropdown').addEventListener('touchstart', search.stickySearch);
                }
                if (!_this.searchCleartext) {
                    _this.clearElementdevice = document.createElement('span');
                    _this.clearElementdevice.innerHTML = 'Clear';
                    domQ(_this.clearElementdevice).addClass('suggestion-item');
                    domQ(_this.clearElementdevice).addClass('navigation-suggestion-item--clear');
                    if (document.querySelector('.navigation-suggestion-item--title')) {
                        document.querySelector('.navigation-suggestion-item--title').appendChild(_this.clearElementdevice);
                    }
                    _this.clearElementdevice.addEventListener('click',
                        function () {
                            _this.localStorage = window.localStorage;
                            search.clearRecentSearches();
                        });
                }
                if (_this.searchCleartext) {
                    _this.searchCleartext.innerText = 'Clear';
                    domQ(_this.searchCleartext).addClass('navigation-suggestion-item--clear');
                }
                if (_this.clearRecentSearchText) {
                    _this.clearRecentSearchText.innerText = '';
                }

                //IPAD 767 landscape to portrait
                if (/iPad/.test(userAgent) && (window.innerWidth === 768 || window.innerWidth === 767)) {
                    _this.mnslogo = document.querySelector('.navigation-logo');
                    _this.ipadinput = document.querySelector('.navigation-dropdown__panel--search');
                    _this.Ipadresize = document.querySelector('.navigation-res-nav-header__search .navigation-search-bar--search-wrap');
                    domQ(_this.mnslogo).addClass('navigation-logo-ipad-portrait');
                    domQ(_this.ipadinput).addClass('navigation-search-bar-ipad-portrait');
                    domQ(_this.Ipadresize).addClass('navigation-search-warp-ipad-portrait');
                    domQ(_this.headerSection).addClass('navigation-header-ipad-height');
                }
            }
            var recentSearchLength = document.querySelectorAll('.suggestion-item.navigation-suggestion-item--recent').length;
            var clearElement = document.querySelector('.navigation-suggestion-item--clear');
            var clearTextElement = document.querySelector('.navigation-suggestion-item--clear-text');

            // search overlay design for desktop breakpoint
            if ((Breakpoint.size() === 'medium' || Breakpoint.size() === 'large')) {
                if (clearTextElement) {
                    clearTextElement.innerHTML = CLEAR;
                }

                if (recentSearchLength === 1) {
                    clearElement.style.top = '-11rem';
                } else if (recentSearchLength === 2) {
                    clearElement.style.top = '-15rem';
                }

                var recentElement = document.querySelector('.navigation-suggestion-item--title'),
                    suggestionListArray = document.querySelectorAll('.navigation-search-item--list li.suggestion-item'),
                    suggestionList = document.querySelector('.navigation-search-item--list');
                if (suggestionListArray.length !== 0) {
                    if (recentElement) {
                        domQ(suggestionList).addClass('navigation-recent-serach-underline');
                    }
                }
                else if (suggestionListArray.length === 0) {
                    if (recentElement) {
                        domQ(suggestionList).removeClass('navigation-recent-serach-underline');
                    }
                }
                if (recentElement) {
                    var recentSearchList = document.querySelector('.navigation-recentsearch--list');
                    recentSearchList.style.maxHeight = '1000px';
                    recentSearchList.style.opacity = 1;
                    recentElement.style.maxHeight = '1000px';
                    recentElement.style.opacity = 1;
                }
            }
            else {
                if (clearTextElement) {
                    clearTextElement.innerHTML = CLR_RECENT_SEARCHES_MD;
                }
            }
        }


        function clearElmntDevice() {
            _this.localStorage = window.localStorage;
            search.clearRecentSearches();
        }

        // Ipad portrait to land scape responsive code --End here

        var body = document.querySelector('body');

        function firstTab(event) {
            var keyPressed = event.key;
            var logo = document.querySelector('.navigation-mns-logo'),
                bodyEl = document.querySelector('body'),
                headerEl = document.querySelector('#common-hdr-main');
            if ((keyPressed === 'Tab' || keyPressed === 9) && !window.l1focus && event.target === bodyEl && !domQ(headerEl).hasClass('navigation-sticky')) {
                if (document.activeElement !== logo && document.querySelector('.navigation-search-overlay-hidden')) {
                    logo.focus();
                }
                body.removeEventListener('keyup', firstTab);
                body.removeEventListener('click', activeElementListener);
            }
        }

        function activeElementListener() {
            window.activeElement = document.activeElement;
        }

        function enableSyteCdn() {
            var headEl = document.getElementsByTagName('head')[0];
            var script = document.createElement('script');
            script.type = 'text/javascript';

            if (Breakpoint.size() === 'xsmall' && window.MS_COMMON_HDR_VISUAL_SEARCH_ENABLE) {
                script.src = '//cdn.syteapi.com/assets/imajs/imajs.js?a=6783&sig=sNbK4CD+LKKs74XPOJOm73478zXaQoRKGQHHF4PTtUk=';
                script.setAttribute('defer', '');
                headEl.appendChild(script);
            }
        }

        function touchHandlerForDropdown(e) {
            if (e.target) {
                if ((e.target.className !== 'navigation-res-nav-header__item navigation-res-nav-header__item--login' || e.target.className === '') &&
                    e.target.parentNode.className !== 'navigation-dropdown__list-container-signin') {
                    domQ(document.querySelector('.navigation-signin-wrapper')).addClass('navigation-signin-wrapper-hidden');
                }
                if ((e.target.className !== 'navigation-res-nav-header__item navigation-res-nav-header__item--myaccount' || e.target.className === '') &&
                    e.target.parentNode.className !== 'navigation-dropdown__list-container-myaccount') {
                    domQ(document.querySelector('.navigation-signacc-wrapper')).addClass('navigation-signacc-wrapper-hidden');
                }
                if ((domQ(document.querySelector('.navigation-minibag-dock-overlay')).hasClass('navigation-dock-overlay--block') ||
                    e.target.parentNode.className !== 'navigation-dock-overlay__inner' ||
                    e.target.parentNode.className !== 'navigation-hdr-mini-bag__item-detail')) {
                    var hasBagOverlayParent = hasSomeParentTheClass(e.target.parentNode, 'navigation-dock-overlay__inner');
                    if (!hasBagOverlayParent) {
                        domQ(document.querySelector('.navigation-minibag-dock-overlay')).removeClass('navigation-dock-overlay--block');
                        domQ(document.querySelector('.navigation-minibag-dock-overlay')).removeClass('navigation-dock-overlay--transition');
                    }
                }
                var userAgent = window.navigator.userAgent || window.navigator.vendor || window.opera;
                if (/iPad/.test(userAgent)) {
                    if ((e.target.className === 'navigation-primary__menu-icon navigation-icon--my-account' || e.target.className === 'navigation-res-nav-header__login-link') &&
                        (e.target.parentNode.className === 'navigation-res-nav-header__login-link' || e.target.parentNode.className === 'navigation-res-nav-header__item navigation-res-nav-header__item--login')) {
                        domQ(document.querySelector('.navigation-signin-wrapper')).removeClass('navigation-signin-wrapper-hidden');
                        domQ(document.querySelector('.navigation-header__signin')).addClass('navigation-header__sign-in');
                    }
                    if ((e.target.className === 'navigation-primary__menu-icon navigation-icon--account-active' || e.target.className === 'navigation-res-nav-header__icon--my-account') &&
                        (e.target.parentNode.className === 'navigation-res-nav-header__icon--my-account' || e.target.parentNode.className === 'navigation-res-nav-header__item navigation-res-nav-header__item--myaccount navigation-myaccount navigation-dropdown')) {
                        domQ(document.querySelector('.navigation-signacc-wrapper')).removeClass('navigation-signacc-wrapper-hidden');
                        domQ(document.querySelector('.navigation-header__sub-list-container--signacc')).addClass('navigation-header__sub-list-container--signacc-displayblock');
                    }
                }
            }
        }

        // returns true if the element or one of it's parents has the given className
        function hasSomeParentTheClass(element, classname) {
            if (element && element.className && element.className.split(' ').indexOf(classname) >= 0) {
                return true;
            }
            return element.parentNode && hasSomeParentTheClass(element.parentNode, classname);
        }

        window.addEventListener('scroll', navStickyScrollEvent);
        var lastScrollTop = 0;
        var hamburgerMenu = document.querySelector('.navigation-primary__section');
        var userAgent = window.navigator.userAgent || window.navigator.vendor || window.opera;
        var HeaderSticked = false;
        function navStickyScrollEvent() {
            if (/iPhone|iPod/.test(userAgent) && !window.MSStream) {
                if (domQ(hamburgerMenu).hasClass('navigation-is-active')) {
                    window.scrollTo(0, window.initialStickyScroll);
                    return false;
                }
            }

            if (!window.MS_COMMON_HDR_STICKY || userAgent.indexOf('MSIE') !== -1 || window.navigator.appVersion.indexOf('Trident/') > 0) {
                return false;
            }

            var pageType = document.getElementById('pageType') ? document.getElementById('pageType').innerText : false;
            var currentScrollTop = window.pageYOffset || document.documentElement.currentScrollTop;
            var headerEl = document.querySelector('#common-hdr-main');
            var stickyHeaderDiv = document.querySelector('.navigation-tostick');
            var stickyFilterPLP = document.querySelector('.container--plp .sticky');
            var srpSticky = document.querySelector('.search-result-content .sticky');
            this.stickyAddtoCartPDP = document.querySelector('.sticky-add-to-bag');
            var findPages = document.querySelector('.filter-sticky-bar.filter-sticky-bar-sticky');
            if (pageType === 'PLP' && stickyFilterPLP) {
                stickyFilterPLP.style.transition = 'padding-top .5s ease';
            }
            if (pageType === 'SRP' && srpSticky) {
                srpSticky.style.transition = 'padding-top .5s ease';
            }
            if (findPages) {
                findPages.style.transition = 'padding-top .5s ease';
            }

            function addStickyToHeader() {
                if (domQ(headerEl).hasClass('navigation-sticky-in') && !domQ(headerEl).hasClass('navigation-sticky')) {
                    domQ(headerEl).removeClass('navigation-sticky-in');
                    HeaderSticked = false;
                    return false;
                }
                if (!HeaderSticked) {
                    headerEl.style.height = headerEl.offsetHeight + 'px';
                    domQ(headerEl).addClass('navigation-sticky');
                    setTimeout(function () {
                        domQ(headerEl).addClass('navigation-sticky-in');
                    }, 10);
                    if (this.stickyAddtoCartPDP && !domQ(this.stickyAddtoCartPDP).hasClass('navigation-sticky-pdp-top')) {
                        domQ(this.stickyAddtoCartPDP).addClass('navigation-sticky-pdp-top');
                    }
                    if (userAgent.indexOf('MSIE') !== -1 || window.navigator.appVersion.indexOf('Trident/') > 0) {
                        var evt = document.createEvent('UIEvents');
                        evt.initUIEvent('resize', true, false, window, 0);
                        window.dispatchEvent(evt);
                    } else {
                        window.dispatchEvent(new Event('resize')); // eslint-disable-line no-undef
                    }
                    stickyHeader.bindAccessibility(document.activeElement);
                    setTimeout(function () {
                        stickyHeaderDiv.style.transition = 'top .65s ease';
                    }, 500);
                    HeaderSticked = true;
                }
            }

            function removeStickyHeader() {
                if (HeaderSticked) {
                    if (domQ(headerEl).hasClass('navigation-sticky')) {
                        domQ(headerEl).removeClass('navigation-sticky-in');
                        setTimeout(function () {
                            domQ(headerEl).removeClass('navigation-sticky');
                            headerEl.removeAttribute('style');
                            stickyHeaderDiv.removeAttribute('style');
                            if (userAgent.indexOf('MSIE') !== -1 || window.navigator.appVersion.indexOf('Trident/') > 0) {
                                var evt = document.createEvent('UIEvents');
                                evt.initUIEvent('resize', true, false, window, 0);
                                window.dispatchEvent(evt);
                            } else {
                                window.dispatchEvent(new Event('resize')); // eslint-disable-line no-undef
                            }
                        }, 500);
                    }
                    if (this.stickyAddtoCartPDP && domQ(this.stickyAddtoCartPDP).hasClass('navigation-sticky-pdp-top')) {
                        domQ(this.stickyAddtoCartPDP).removeClass('navigation-sticky-pdp-top');
                    }
                    if (userAgent.indexOf('MSIE') !== -1 || window.navigator.appVersion.indexOf('Trident/') > 0) {
                        var evt = document.createEvent('UIEvents');
                        evt.initUIEvent('resize', true, false, window, 0);
                        window.dispatchEvent(evt);
                    } else {
                        window.dispatchEvent(new Event('resize')); // eslint-disable-line no-undef
                    }
                    stickyHeader.unbindAccessibility();
                    HeaderSticked = false;
                }
            }

            function removeStickyHeaderWithoutDelay() {
                if (HeaderSticked) {
                    headerEl.removeAttribute('style');
                    if (domQ(headerEl).hasClass('navigation-sticky')) {
                        domQ(headerEl).removeClass('navigation-sticky-in');
                        setTimeout(function () {
                            domQ(headerEl).removeClass('navigation-sticky');
                            headerEl.removeAttribute('style');
                            stickyHeaderDiv.removeAttribute('style');
                            if (userAgent.indexOf('MSIE') !== -1 || window.navigator.appVersion.indexOf('Trident/') > 0) {
                                var evt = document.createEvent('UIEvents');
                                evt.initUIEvent('resize', true, false, window, 0);
                                window.dispatchEvent(evt);
                            } else {
                                window.dispatchEvent(new Event('resize')); // eslint-disable-line no-undef
                            }
                        }, 10);
                    }
                    if (this.stickyAddtoCartPDP && domQ(this.stickyAddtoCartPDP).hasClass('navigation-sticky-pdp-top')) {
                        domQ(this.stickyAddtoCartPDP).removeClass('navigation-sticky-pdp-top');
                    }
                    if (userAgent.indexOf('MSIE') !== -1 || window.navigator.appVersion.indexOf('Trident/') > 0) {
                        var evt = document.createEvent('UIEvents');
                        evt.initUIEvent('resize', true, false, window, 0);
                        window.dispatchEvent(evt);
                    } else {
                        window.dispatchEvent(new Event('resize')); // eslint-disable-line no-undef
                    }
                    stickyHeader.unbindAccessibility();
                    HeaderSticked = false;
                }
            }


            function stickyCondition() {
                if (currentScrollTop > lastScrollTop) {
                    removeStickyHeader();
                    lastScrollTop = currentScrollTop <= 0 ? 0 : currentScrollTop; // For Mobile or negative scrolling
                } else {
                    if (lastScrollTop - currentScrollTop > 100) {
                        addStickyToHeader();
                        lastScrollTop = currentScrollTop <= 0 ? 0 : currentScrollTop; // For Mobile or negative scrolling
                    }
                }
            }
            if (window.navSkiptoPage === true) {
                removeStickyHeaderWithoutDelay();
                window.navSkiptoPage = false;
            }
            if (currentScrollTop > 600) {
                if (headerEl && headerEl.style.display !== 'none') {
                    stickyCondition();
                }
            }
            if (currentScrollTop < 600) {
                lastScrollTop = 0;
                if (headerEl && headerEl.style.display !== 'none') {
                    removeStickyHeaderWithoutDelay();
                }
            }
            if (!currentScrollTop) {
                removeStickyHeaderWithoutDelay();
            }
        }
    }

    return Header;
});
