'use strict';
define([
    'xhr-promise',
    '../links/cookieService'
], function (XhrPromise, mnsCookieService) {

    var loyaltyService;

    // loyalty service API
    loyaltyService = {
        /**
         * Validates if environment supports CORS
         * @returns {Boolean} : TRUE if environment supports CORS
         */
        fullySupportsCORS: function (HttpRequestConstructor) {
            var support = false;

            HttpRequestConstructor = HttpRequestConstructor || XMLHttpRequest;

            if (HttpRequestConstructor) {
                var request = new HttpRequestConstructor();
                if (typeof request.withCredentials !== 'undefined') {
                    support = true;
                }
            }
            return support;
        },

        /**
         * Initiates web request to load loyalty offers
         * @param {String} loyaltyOfferApi
         * @param {String} authToken
         */
        getOffers: function(loyaltyOfferApi, authToken) {

            return this.request({
                url: loyaltyOfferApi,
                method: 'GET',
                headers: {
                    'Authorization': 'MNSAuthToken ' + authToken,
                    'Accept': 'application/vnd.mns.ls-v3+json, application/json'
                }
            }).then(function (response) {
                if (response.status === 200) {
                    return response.responseText;
                }
                //We might want to log this error. Error thrown so that the page redirects to sparks landing page.
                throw new Error('Offers cannot be obtained');
            });

        },

        /**
         * auth token promise object to be resolved with cookie value
         */
        getAuthToken: function(authTokenUrl) {
            var token = mnsCookieService.getAuthTokenCookieValue();

            if (token === null) {
                return this.getAuthTokenFromServer(authTokenUrl);
            }

            return new Promise(function (resolve) {
                return resolve(token);
            });
        },

        /**
         * auth token promise object to be resolved with server response
         */
        getAuthTokenFromServer: function(authTokenUrl) {

            return this.request({
                url: authTokenUrl + '?refresh=true',
                method: 'GET'
            }).then(function (response) {
                if (response.status === 200) {
                    return response.responseText.token;
                }
                //We might want to log this error. Error thrown so that the page redirects to sparks landing page.
                throw new Error('User cannot be authorised');
            });
        },

        /**
         * xhr-promise request promise
         */
        request: function(options) {
            var xhrp = new XhrPromise();
            return xhrp.send(options);
        }
    };

    return loyaltyService;
});
