'use strict';

define(['./../util', '../../../../lib/domQ', './../Constant', './CountryLinks', './../stickyHeader'], function (util, domQ, Constant, countryLinkDropDown, stickyHeader) {
    var IS_ACTIVE = 'navigation-is-active';
    var IS_SELECTED = 'is-selected';
    var PREVENT_SCROLLING = 'navigation-prevent-scrolling';
    var PREVENT_STICKYSCROLLING = 'navigation-prevent-stickyscrolling';
    var STICKY_CLASS = 'navigation-sticky';
    var ANIMATION_TRANSITION = 'transform .1s cubic-bezier(0, 0.3, 0.2, 1.02) 0s';
    var L1_ANIMATION_TRANSITION = 'transform .1s cubic-bezier(0, 0.3, 0.2, 1.02) 0s';
    var HAMBURGER_CLASS = 'navigation-res-nav-header__item--menu';
    var TOUCH_START = 'touchstart';
    var PRIMARY_SECTION_WIDTH = 0.9;
    var L3_MAXHEIGHT = '1000px';

    function XSmallDevice(level2) {
        this.level2 = level2;
        this.navPrimarySectionEl = document.querySelector('.navigation-primary__section');
        this.navigationOverlayEl = document.getElementById('navigationOverlay');
        this.navContainerEl = document.getElementById('navContainer');
        this.hamburgerEl = document.querySelector('.' + HAMBURGER_CLASS);
        this.backLabelEl = document.querySelector('.navigation-submenu__back-bar-label');
        this.backButtonEl = document.querySelector('.navigation-submenu__back-bar-link');
        this.bodyEl = document.querySelector('body');
        this.level2Headers = document.querySelectorAll('#navContainer .nav-submenu__l2-header');
        this.headerEl = document.querySelector('#progressiveHeaderSection');
        this.hamburgerNavCloseButton = document.querySelector('.navigation-burger-close');
        this.navVeil = document.querySelector('#progressiveHeaderSection > div.navigation-nav-veil');
        this.searchButtonReset = document.querySelector('.navigation-search-bar--clear');
        this.searchiconButton = document.querySelector('.navigation-search-bar--submit');
        this.searchBar = document.querySelector('.navigation-search-bar--search');
        this.searchBarSuggession = document.querySelector('.navigation-search-bar--suggestion-dropdown');
        this.visualSearch = document.querySelector('.navigation-hdr-image-search');
        this.navPrimaryListEl = document.querySelectorAll('.nav-primary__list li');
        this.navBottomAccountIcon = document.querySelectorAll('.navigation-bottom-myaccount>a>i.navigation-primary__menu-icon.navigation-icon--my-account');
        this.navBottomMyAccount = document.querySelectorAll('.navigation-bottom-myaccount');
        this.navBottomSparksIcon = document.querySelectorAll('.navigation-bottom-sparks');
        this.navBottomFindStore = document.querySelectorAll('.navigation-bottom--find-store');
        this.navBottomFindStoreIcon = document.querySelectorAll('.navigation-bottom--find-store>a>i.navigation-mobile-location-finder');
        this.mobileErrorPrompt = document.querySelector('.navigation-overlay-error');
        this.errorPromptCloseBtn = document.querySelector('.navigation-error-close-btn');
        this.errorPromptPopup = document.querySelector('.navigation-error-popup');
        this.navPrimaryList = document.querySelectorAll('.nav-primary__list');
        this.navBottomContainerOne = document.querySelector('.navigation-pane-one-navbottom');
        this.navBottomContainerTwo = document.querySelector('.navigation-pane-two-navbottom');
        this.imageSearchBalloon = document.querySelector('.navigation-hdr-imagesearch--balloon');
        this.imageSearchBalloonClose = document.querySelector('.navigation-hdr-balloon--clear');
        this.navPrimaryFixedTopBar = document.querySelector('.navigation-burger__fixedtopmenu');
        this.goToCountryLink = document.querySelector('.navigation-country-mobile--link');
        this.mobileNavLogo = document.querySelector('.navigation-burger__fixedtopmenu .navigation-logo');
        this.headerEL = document.getElementById('common-hdr-main');
        this.countrySelectorHead = document.querySelector('.navigation-mobile-country-header');
        this.countrySelector = document.querySelector('.navigation-mobile-country-selector');
        this.mobileCountryDiv = document.querySelector('.navigation-nav-country-selector');
        this.userAgent = window.navigator.userAgent || window.navigator.vendor || window.opera;

        // bind functions for event listeners
        this.level2Click = this.level2Click.bind(this);
        this.toggleCountrySelectorDiv = this.toggleCountrySelectorDiv.bind(this);
        this.searchButtonResetClick = this.searchButtonResetClick.bind(this);
        this.mobileMenuToggle = this.mobileMenuToggle.bind(this);
        this.backClick = this.backClick.bind(this);
        this.visualSearchClick = this.visualSearchClick.bind(this);
        this.errorPromptCloseBtnClick = this.errorPromptCloseBtnClick.bind(this);
        this.errorPromptPopupClick = this.errorPromptPopupClick.bind(this);
        this.toggleLevel3 = this.toggleLevel3.bind(this);
        this.toggleLevel3Keyup = this.toggleLevel3Keyup.bind(this);
        this.closeMenu = this.closeMenu.bind(this);
        this.visualSearchIcon = this.visualSearchIcon.bind(this);
        this.checkHeaderClickToClose = this.checkHeaderClickToClose.bind(this);
        this.myAccount = this.myAccount.bind(this);
        this.interactionEvent = 'click';
        this.goToCountryLinkClick = this.goToCountryLinkClick.bind(this);
        this.mnsHeaderConfig = window.mnsHeaderConfig;
        this.imageSearchBalloonToggle = this.imageSearchBalloonToggle.bind(this);
        if (util.hasTouchCapability()) {
            this.interactionEvent = TOUCH_START;
        }
    }
    XSmallDevice.prototype.imageSearchBalloonToggle = function (e) {
        if (!util.hasClass(this.imageSearchBalloon, 'navigation-hdr-imagesearch--balloon__hidden')) {
            domQ(this.imageSearchBalloon).addClass('navigation-hdr-imagesearch--balloon__hidden');
            e.preventDefault();
        }
    };
    XSmallDevice.prototype.searchButtonResetClick = function () {
        domQ(this.searchBarSuggession).addClass('navigation-search-bar--suggestion-dropdown__hidden');
        domQ(this.searchiconButton).removeClass('navigation-search-bar--submit__active');
        this.searchBar.value = '';
        this.visualSearch ? this.displayNode(this.visualSearch, true) : '';
        this.errorPromptCloseBtnClick();
    };
    XSmallDevice.prototype.visualSearchClick = function () {
        if (!(window.SyteApp && window.SyteApp.isReady)) {
            this.displayNode(this.mobileErrorPrompt, true);
        }
    };
    XSmallDevice.prototype.errorPromptCloseBtnClick = function (e) {
        if (this.mobileErrorPrompt.style.display === 'block') {
            this.displayNode(this.mobileErrorPrompt, false);
            e.preventDefault();
        }
    };
    XSmallDevice.prototype.errorPromptPopupClick = function (e) {
        e.preventDefault();
    };
    XSmallDevice.prototype.bind = function () {
        var _this = this;
        for (var iCount = 0; iCount < this.level2.navPrimaryAList.length; iCount++) {
            var link = this.level2.navPrimaryAList[iCount];
            link.addEventListener(_this.interactionEvent, _this.level2Click);
            if (_this.interactionEvent === TOUCH_START) {
                link.removeEventListener('touchstart', _this.level2Click);
                link.addEventListener('click', _this.level2Click);
            }
        }

        for (var navPrimaryListElCount = 0; navPrimaryListElCount < this.navPrimaryListEl.length; navPrimaryListElCount++) {
            var navLiEl = this.navPrimaryListEl[navPrimaryListElCount];
            if (!domQ(navLiEl).hasClass('nav-primary-inspire-me')) {
                navLiEl.addEventListener(_this.interactionEvent, _this.level2Click);
                if (_this.interactionEvent === TOUCH_START) {
                    navLiEl.removeEventListener('touchstart', _this.level2Click);
                    navLiEl.addEventListener('click', _this.level2Click);
                }
            }
        }

        this.hamburgerEl.addEventListener('click', this.mobileMenuToggle);
        if (this.visualSearch) {
            this.visualSearch.addEventListener('click', this.visualSearchClick);
        }
        if (this.errorPromptCloseBtn) {
            this.errorPromptCloseBtn.addEventListener('touchstart', this.errorPromptCloseBtnClick);
        }
        if (this.errorPromptPopup) {
            this.errorPromptPopup.addEventListener('touchstart', this.errorPromptPopupClick);
        }
        this.backButtonEl.addEventListener('click', this.backClick);
        this.searchButtonReset.addEventListener('click', this.searchButtonResetClick);
        this.searchBar.addEventListener(this.interactionEvent, this.closeMenu);
        this.searchBar.addEventListener('input', this.visualSearchIcon);
        if (this.imageSearchBalloonClose) {
            this.imageSearchBalloonClose.addEventListener('touchstart', this.imageSearchBalloonToggle);
        }
        for (var iHeader = 0; iHeader < this.level2Headers.length; iHeader++) {
            var header = this.level2Headers[iHeader];
            header.addEventListener('click', _this.toggleLevel3);
            header.addEventListener('keyup', _this.toggleLevel3Keyup);
        }
        this.navigationOverlayEl.addEventListener(this.interactionEvent, this.closeMenu);
        this.goToCountryLink.addEventListener(this.interactionEvent, this.goToCountryLinkClick);
        //close the hamburger on click event only
        this.headerEl.addEventListener('click', this.checkHeaderClickToClose);
        this.countrySelectorHead.addEventListener('click', this.toggleCountrySelectorDiv);
        this.hamburgerNavCloseButton.addEventListener('click', this.checkHeaderClickToClose);
        this.setup();
        if (this.searchBar.value !== '') {
            this.displayNode(this.visualSearch, false);
        }
        //for safari-bfcache clear
        window.onpageshow = function (event) {
            if (event.persisted) { //from bf cache
                window.location.reload();
            }
        };
        if (this.mnsHeaderConfig && this.mnsHeaderConfig.storeId && this.mnsHeaderConfig.storeId === Constant.ieId) {
            this.setNavBottomContainerIE();
        }
        this.setPrimarySection();
    };
    XSmallDevice.prototype.unbind = function () {
        var _this = this;
        for (var iCount = 0; iCount < this.level2.navPrimaryAList.length; iCount++) {
            var link = this.level2.navPrimaryAList[iCount];
            link.removeEventListener(_this.interactionEvent, _this.level2Click);
            if (_this.interactionEvent === TOUCH_START) {
                link.removeEventListener('click', util.preventClick);
            }
        }
        for (var navPrimaryListElCount = 0; navPrimaryListElCount < this.navPrimaryListEl.length; navPrimaryListElCount++) {
            var navLiEl = this.navPrimaryListEl[navPrimaryListElCount];
            if (!domQ(navLiEl).hasClass('nav-primary-inspire-me')) {
                navLiEl.removeEventListener(_this.interactionEvent, _this.level2Click);
                if (_this.interactionEvent === TOUCH_START) {
                    navLiEl.removeEventListener('click', _this.level2Click);
                }
            }
        }
        this.hamburgerEl.removeEventListener('click', this.mobileMenuToggle);
        if (this.visualSearch) {
            this.visualSearch.removeEventListener('click', this.visualSearchClick);
        }
        if (this.errorPromptCloseBtn) {
            this.errorPromptCloseBtn.removeEventListener('touchstart', this.errorPromptCloseBtnClick);
        }
        if (this.errorPromptPopup) {
            this.errorPromptPopup.removeEventListener('touchstart', this.errorPromptPopupClick);
        }
        this.backButtonEl.removeEventListener('click', this.backClick);
        for (var iHeader = 0; iHeader < this.level2Headers.length; iHeader++) {
            var header = this.level2Headers[iHeader];
            header.removeEventListener('click', _this.toggleLevel3);
            header.removeEventListener('keyup', _this.toggleLevel3Keyup);
        }
        this.navigationOverlayEl.removeEventListener(this.interactionEvent, this.closeMenu);
        this.headerEl.removeEventListener('click', this.checkHeaderClickToClose);
        this.hamburgerNavCloseButton.removeEventListener('click', this.checkHeaderClickToClose);
        if (this.imageSearchBalloonClose) {
            this.imageSearchBalloonClose.removeEventListener('touchstart', this.imageSearchBalloonToggle);
        }
        this.teardown();
    };
    XSmallDevice.prototype.toggleCountrySelectorDiv = function () {
        countryLinkDropDown.buildDropDown();
        countryLinkDropDown.resetDropDown();
        if (domQ(this.countrySelector).hasClass('navigation-country-overlay-visible')) {
            domQ(this.countrySelector).removeClass('navigation-country-overlay-visible');
        }
        else {
            this.navPrimarySectionEl.style.overflowY = 'hidden';
            domQ(this.countrySelector).addClass('navigation-country-overlay-visible');
            var parentEl = this.navPrimarySectionEl;
            var countryDiv = this.mobileCountryDiv;
            setTimeout(function () {
                parentEl.scrollTop = countryDiv.offsetTop;
                parentEl.style.overflowY = 'scroll';
            }, 500);
        }
        if (domQ(this.countrySelectorHead).hasClass('navigation-country-toggle-arrow')) {
            domQ(this.countrySelectorHead).removeClass('navigation-country-toggle-arrow');
            countryLinkDropDown.resetDropDown();
        }
        else {
            domQ(this.countrySelectorHead).addClass('navigation-country-toggle-arrow');
        }
    };
    XSmallDevice.prototype.goToCountryLinkClick = function () {
        if (this.goToCountryLink.href === 'javascript:;') {
            this.closeMenu();
        }
    };
    XSmallDevice.prototype.setup = function () {
        var _this = this;
        this.myAccount();
        if (this.navContainerEl.classList.contains(IS_ACTIVE)) { //close menu only if it is open
            this.backClick();
        }
        clearTimeout(this.setTimeoutId);
        this.setTimeoutId = setTimeout(function () {
            _this.addAnimation();
        }, 100);
    };
    XSmallDevice.prototype.teardown = function () {
        this.level2.deactivate();
        this.removeAnimation();
        this.closeMenu();
    };
    XSmallDevice.prototype.addAnimation = function () {
        this.navPrimarySectionEl.style.transition = ANIMATION_TRANSITION;
        this.navPrimaryFixedTopBar.style.transition = ANIMATION_TRANSITION;
    };
    XSmallDevice.prototype.removeAnimation = function () {
        this.navPrimarySectionEl.style.transition = 'none';
        this.navPrimaryFixedTopBar.style.transition = 'none';
    };
    XSmallDevice.prototype.myAccount = function () {
        var isAuthenticated = this.level2.auth.isAuthenticated();
        var mobileAccountEl = document.querySelector('.navigation-bottom-container');
        this.displayNode(mobileAccountEl.children[1].children[0].children[2], isAuthenticated);
        this.displayNode(mobileAccountEl.children[1].children[0].children[3], !isAuthenticated);
        this.level2MyAccount();
    };
    XSmallDevice.prototype.level2MyAccount = function () {
        var isAuthenticated = this.level2.auth.isAuthenticated();
        var mobileAccountEl = document.querySelector('.navigation-pane-two-navbottom > .navigation-bottom-container');
        this.displayNode(mobileAccountEl.children[1].children[0].children[2], isAuthenticated);
        this.displayNode(mobileAccountEl.children[1].children[0].children[3], !isAuthenticated);
    };
    XSmallDevice.prototype.displayNode = function (el, show) {
        el.style.display = show ? 'block' : 'none';
    };
    XSmallDevice.prototype.level2Open = function (el) {
        this.isActiveBottomContainer();
        //If Span is replaced with <img> in future change the below code
        if (el.tagName === 'DIV' && el.parentElement && !el.parentElement.classList.contains(IS_ACTIVE)) {
            this.level2.activate(el.parentElement);
            this.setBackLabel(el.parentElement.innerHTML, el.parentElement.pathname);
        } else if (el.tagName === 'IMG' && el.parentElement.parentElement && !el.parentElement.parentElement.classList.contains(IS_ACTIVE)) {
            this.level2.activate(el.parentElement.parentElement);
            this.setBackLabel(el.parentElement.parentElement.innerHTML, el.parentElement.parentElement.pathname);
        } else if (el.tagName === 'IMG' && el.parentElement.parentElement) {
            this.setBackLabel(el.parentElement.parentElement.innerHTML, el.parentElement.parentElement.pathname);
        } else if (el.tagName === 'LI' && el.firstElementChild && !el.firstElementChild.classList.contains(IS_ACTIVE)) {
            this.level2.activate(el.firstElementChild);
            this.setBackLabel(el.firstElementChild.innerHTML, el.firstElementChild.pathname);
        } else {
            if (!domQ(el).hasClass(IS_ACTIVE)) {
                this.level2.activate(el);
                this.setBackLabel(el.innerHTML, el.pathname);
            }
        }
    };
    XSmallDevice.prototype.isActiveBottomContainer = function () {
        if (this.navBottomContainerOne && domQ(this.navBottomContainerOne).hasClass(IS_ACTIVE)) {
            domQ(this.navBottomContainerOne).removeClass(IS_ACTIVE);
        }
        if (this.navBottomContainerTwo && !domQ(this.navBottomContainerTwo).hasClass(IS_ACTIVE)) {
            domQ(this.navBottomContainerTwo).addClass(IS_ACTIVE);
        }
    };
    XSmallDevice.prototype.level2Click = function (e) {
        e.preventDefault();
        this.level2Open(e.target);
        return false;
    };
    XSmallDevice.prototype.mobileMenuToggle = function () {
        if (this.navPrimarySectionEl.classList.contains(IS_ACTIVE)) {
            this.closeMenu();
        } else {
            this.openMenu();
        }
    };
    XSmallDevice.prototype.openMenu = function () {
        if (domQ(this.headerEL).hasClass(STICKY_CLASS)) {
            domQ(this.bodyEl).addClass(PREVENT_STICKYSCROLLING);
            domQ(document.querySelector('html')).addClass(PREVENT_STICKYSCROLLING);
        } else {
            domQ(this.bodyEl).addClass(PREVENT_SCROLLING);
        }
        domQ(this.navPrimarySectionEl).addClass(IS_ACTIVE);
        domQ(this.navPrimaryFixedTopBar).addClass(IS_ACTIVE);
        domQ(this.navBottomContainerOne).addClass(IS_ACTIVE);
        domQ(this.hamburgerEl).addClass(IS_SELECTED);
        domQ(this.navVeil).addClass(IS_ACTIVE);
        this.mobileNavLogo.style.transform = 'none';
        this.addAnimation();
        this.anchoredNavigation();
    };
    XSmallDevice.prototype.closeMenu = function () {
        util.removeClass(this.bodyEl, PREVENT_STICKYSCROLLING);
        util.removeClass(document.querySelector('html'), PREVENT_STICKYSCROLLING);
        util.removeClass(this.bodyEl, PREVENT_SCROLLING);
        this.backClick();
        domQ(this.navPrimarySectionEl).removeClass(IS_ACTIVE);
        domQ(this.navPrimaryFixedTopBar).removeClass(IS_ACTIVE);
        domQ(this.navBottomContainerOne).removeClass(IS_ACTIVE);
        domQ(this.hamburgerEl).removeClass(IS_SELECTED);
        domQ(this.navVeil).removeClass(IS_ACTIVE);
        if (domQ(this.countrySelectorHead).hasClass('navigation-country-toggle-arrow')) {
            this.toggleCountrySelectorDiv();
        }
        this.navBottomContainerOne.style.transition = null;
        this.mobileNavLogo.style.transform = 'translate3d(-105%, 0, 0)';
    };
    XSmallDevice.prototype.toggleLevel3 = function (e) {
        if (e.target.classList.contains(IS_SELECTED)) {
            this.closeLevel3(e.target);
        } else {
            this.openLevel3(e.target);
        }
    };
    XSmallDevice.prototype.toggleLevel3Keyup = function (event) {
        var keyPressed = event.key || event.keyCode;
        if (keyPressed === 13 || keyPressed === 'Enter') {
            if (event.target.classList.contains(IS_SELECTED)) {
                this.closeLevel3(event.target);
            } else {
                this.openLevel3(event.target);
            }
        }
    };
    XSmallDevice.prototype.openLevel3 = function (header) {
        this.lastLevel3Header = header;
        domQ(header).addClass(IS_SELECTED);
        domQ(header.nextElementSibling).addClass('nav-submenu__selectedlink-list');
        header.nextElementSibling.style.maxHeight = L3_MAXHEIGHT;
        let rootEl = header.parentElement.parentElement.parentElement;
        if (header.classList.contains(IS_SELECTED)) {
            if (header.parentElement.children.length === 2) {
                header.offsetTop === 0 ? rootEl.scrollTop = header.offsetTop : rootEl.scrollTop = header.parentElement.offsetTop;
            } else {
                if (header === header.parentElement.children[0]) {
                    rootEl.scrollTop = header.parentElement.offsetTop;
                } else {
                    rootEl.scrollTop = header.offsetTop;
                }
            }
        }
    };

    XSmallDevice.prototype.closeLevel3 = function (header) {
        header.nextElementSibling.style.maxHeight = null;
        domQ(header.nextElementSibling).addClass('nav-submenu__selectedlink-list');
        setTimeout(function () {
            domQ(header).removeClass(IS_SELECTED);
        }, 500);
    };
    XSmallDevice.prototype.closeAllLevel3 = function (iHeader) {
        var iHeaderlength = iHeader.length;
        for (var i = 0; i < iHeaderlength; i++) {
            domQ(iHeader[i]).removeClass(IS_SELECTED);
            iHeader[i].nextElementSibling.style.maxHeight = null;
            domQ(iHeader[i].nextElementSibling).addClass('nav-submenu__selectedlink-list');
        }
    };
    XSmallDevice.prototype.setBackLabel = function (label, url) {
        this.backLabelEl.innerHTML = '<a href="' + url + '">' + label + '</a>';
    };
    XSmallDevice.prototype.backClick = function () {
        var iHeader = document.querySelectorAll('#navContainer .nav-submenu__l2-header.is-selected');
        domQ(this.navContainerEl).removeClass(IS_ACTIVE);
        if (this.navPrimarySectionEl.classList.contains('navigation-is-level2-active')) {
            this.navPrimarySectionEl.style.transition = L1_ANIMATION_TRANSITION;
            this.navBottomContainerOne.style.transition = L1_ANIMATION_TRANSITION;
            this.navPrimarySectionEl.classList.remove('navigation-is-level2-active');
            this.navBottomContainerOne.classList.remove('navigation-is-level2-active');
            this.level2.deactivateAllMenus();
            if (iHeader && iHeader.length > 0) {
                this.closeAllLevel3(iHeader);
            }
        }
        if (!domQ(this.navBottomContainerOne).hasClass(IS_ACTIVE) && this.navPrimarySectionEl.classList.contains(IS_ACTIVE)) {
            domQ(this.navBottomContainerOne).addClass(IS_ACTIVE);
        }
        if (domQ(this.navBottomContainerTwo).hasClass(IS_ACTIVE)) {
            domQ(this.navBottomContainerTwo).removeClass(IS_ACTIVE);
        }
    };
    XSmallDevice.prototype.checkHeaderClickToClose = function (e) {
        var target = e.target;
        var isChildOfHamburgerTrigger = this.hamburgerEl.contains(target);
        if (!(target.classList.contains(HAMBURGER_CLASS) ||
            isChildOfHamburgerTrigger)) {
            this.closeMenu();
        }
    };
    XSmallDevice.prototype.visualSearchIcon = function () {
        if (this.searchBar.value === '') {
            this.displayNode(this.visualSearch, true);
        } else {
            this.displayNode(this.visualSearch, false);
        }
    };
    //anchored navigation
    XSmallDevice.prototype.anchoredNavigation = function () {
        var l1Menu = this.navPrimarySectionEl.querySelector('.navigation-is-active-anchor');
        var pageType = document.getElementById('pageType') ? document.getElementById('pageType').innerText : 'OTH';
        var l3Menu = null,
            l3Url = window.location.pathname;
        var headerElm = document.querySelector('#common-hdr-main');
        var l3 = this.headerEL.querySelector('.navigation-anchor');
        if (l3) {
            l3.classList.remove('navigation-anchor');
        }
        if (l1Menu !== null && pageType !== 'SLP') {
            this.level2Open(l1Menu);
            if (domQ(headerElm).hasClass('navigation-sticky')) {
                stickyHeader.init();
                stickyHeader.bindAnchor();
                document.querySelector('.navigation-submenu__back-bar-label a').focus();
                var headers = document.querySelectorAll('.navigation-open-submenu .nav-submenu__span--four-fifths .nav-submenu__l2-header');
                util.setTabIndex(headers, '0');
                window.stickyActiveL1 = l1Menu;
            }
        }
        if (window.hpl3Url && window.hpl3Url !== null) {
            l3Menu = this.headerEL.querySelector('a[href$="' + window.hpl3Url + '"]');
        } else if ((window.pdpData || window.navAnchorPdpData) && l1Menu !== null) {
            var length = window.pdpData ? window.pdpData.breadcrumb.length : window.navAnchorPdpData.breadcrumb.length;
            for (var category = 2; category < length; category++) {
                l3Url = window.pdpData ? window.pdpData.breadcrumb[category].categoryUrl : window.navAnchorPdpData.breadcrumb[category].pathname;
                if (l3Menu === null) {
                    var urlWithQ = this.headerEL.querySelector('a[href^="' + l3Url + '?"]');
                    var urlWithHash = urlWithQ === null ? this.headerEL.querySelector('a[href$="' + l3Url + '#intid"]') :
                        urlWithQ;
                    l3Menu = urlWithHash === null ? this.headerEL.querySelector('a[href$="' + l3Url + '"]') :
                        urlWithHash;
                }
                if (l3Menu !== null && l3Menu.classList.length !== 0) {
                    l3Menu = null;
                }
            }
        } else if (l3Url.indexOf('/l/') !== -1) {
            var urlWithOutAnalytics = this.headerEL.querySelector('a[href$="' + l3Url + '"]');
            urlWithOutAnalytics = urlWithOutAnalytics === null ? this.headerEL.querySelector('a[href^="' + l3Url + '?"]') : urlWithOutAnalytics;
            l3Menu = urlWithOutAnalytics === null ? this.headerEL.querySelector('a[href^="' + l3Url + '#intid"]') : urlWithOutAnalytics;
            if (l3Menu === null && l3Url.indexOf('-n-') !== -1) {
                l3Url = l3Url.split('-n-')[0];
                l3Menu = this.headerEL.querySelector('a[href$="' + l3Url + '"]');
            }
        }
        if (l3Menu !== null) {
            var header = l3Menu.parentElement.parentElement.previousElementSibling;
            var rootEl = header.parentElement.parentElement.parentElement;
            header.nextElementSibling.style.transition = 'none';
            this.openLevel3(header);
            domQ(l3Menu).addClass('navigation-anchor');
            setTimeout(function () {
                rootEl.scrollTop = l3Menu.parentElement.offsetTop;
                header.nextElementSibling.style.transition = 'max-height .5s cubic-bezier(0.455, 0.03, 0.515, 0.955) 0s';
            }, 30);
        }
    };
    XSmallDevice.prototype.setNavBottomContainerIE = function () {

        for (var sparksI = 0; sparksI < this.navBottomSparksIcon.length; sparksI++) {
            this.navBottomSparksIcon[sparksI].style.display = 'none';
        }
        for (var accI = 0; accI < this.navBottomAccountIcon.length; accI++) {
            this.navBottomAccountIcon[accI].classList.add('navigation-icon--my-account-ie');
        }
        for (var signInI = 0; signInI < this.navBottomFindStoreIcon.length; signInI++) {
            this.navBottomFindStoreIcon[signInI].classList.add('navigation-icon--find-store-ie');
        }
        for (var accIconI = 0; accIconI < this.navBottomMyAccount.length; accIconI++) {
            domQ(this.navBottomMyAccount[accIconI]).removeClass('navigation-bottom-myaccount');
            domQ(this.navBottomMyAccount[accIconI]).addClass('navigation-bottom-myaccount-ie');
        }
        for (var signInIconI = 0; signInIconI < this.navBottomFindStore.length; signInIconI++) {
            domQ(this.navBottomFindStore[signInIconI]).removeClass('navigation-bottom--find-store');
            domQ(this.navBottomFindStore[signInIconI]).addClass('navigation-bottom--find-store-ie');
        }
    };
    XSmallDevice.prototype.setPrimarySection = function () {
        if (this.navPrimarySectionEl && this.navContainerEl) {
            var paneOneEl = document.querySelector('.navigation-pane-one-navbottom');
            if ((window.innerWidth === 768 || window.innerWidth === 767)) {
                this.navPrimarySectionEl.style.width = window.innerWidth * PRIMARY_SECTION_WIDTH - 270 + 'px';
                this.navPrimaryFixedTopBar.style.width = (window.innerWidth * PRIMARY_SECTION_WIDTH) - 290 + 'px';
                this.navContainerEl.style.width = (window.innerWidth * PRIMARY_SECTION_WIDTH) - 290 + 'px';
                paneOneEl.style.width = window.innerWidth * PRIMARY_SECTION_WIDTH - 270 + 'px';
            } else {
                this.navPrimarySectionEl.style.width = window.innerWidth * PRIMARY_SECTION_WIDTH + 'px';
                this.navPrimaryFixedTopBar.style.width = (window.innerWidth * PRIMARY_SECTION_WIDTH) - 20 + 'px';
                this.navContainerEl.style.width = (window.innerWidth * PRIMARY_SECTION_WIDTH) - 20 + 'px';
                paneOneEl.style.width = window.innerWidth * PRIMARY_SECTION_WIDTH + 'px';
            }

        }
    };
    return XSmallDevice;
});
