
import cookieService from '../js/links/cookieService';
import Authentication from '../js/Authentication';

describe('Progressive Header Authentication', function() {

    const COOKIE_KEY = 'AUTH_TEST_COOKIE';
    let documentCookieMock;
    let documentNoCookieMock;

    beforeEach(function() {
        documentCookieMock = {
            cookie: COOKIE_KEY + '=test; expires=Thu, 01 Jan 1970 00:00:01 GMT;'
        };
        documentNoCookieMock = {
            cookie: ''
        };
    });

    afterEach(function() {
        cookieService.getUserLoginCookieValue.restore();
    });

    it('should authenticate when cookie value exists', function() {
        let testAuth;

        sinon.stub(cookieService, 'getUserLoginCookieValue').returns('test');
        testAuth = new Authentication();

        expect(testAuth.isAuthenticated()).to.equal(true);
    });

    it('should not authenticate when cookie value does not exist', function() {
        let testAuth;

        sinon.stub(cookieService, 'getUserLoginCookieValue').returns(null);
        testAuth = new Authentication();

        expect(testAuth.isAuthenticated()).to.equal(false);
    });

    it('should get cookie value', function() {
        let testAuth;

        sinon.stub(cookieService, 'getUserLoginCookieValue').returns('test');
        testAuth = new Authentication();

        expect(testAuth.getCookieValue()).to.equal('test');
    });

    it('should get null if cookie value does not exist', function() {
        let testAuth;

        sinon.stub(cookieService, 'getUserLoginCookieValue').returns(null);
        testAuth = new Authentication();

        expect(testAuth.getCookieValue()).to.equal(null);
    });

});