import DropdownMenu from '../js/DropdownMenu';

describe('Dropdown Menu Component', function() {

    var dropdownMenuEl,
        dropdown,
        options = {
            menuItem: '.menuitem',
            menuItemLink: '.menuitem-link',
            menuDropdown: '.dropdown-wrapper'
        };

    beforeEach(function() {
        dropdownMenuEl = getDropdownMenuDOM();
        dropdown = new DropdownMenu(dropdownMenuEl, options);
    });

    it('should initialize DropdownMenu component with correct parameters', function() {
        expect(dropdown.menubar).to.equal(dropdownMenuEl);
        expect(dropdown.menuItemLinks.length).to.equal(2);
        expect(dropdown.options).to.equal(options);
    });

    it('should add menuitems to DropdownMenu component', function() {
        var el = document.createElement('div'),
            dropdown = new DropdownMenu(el, options);

        dropdown.addMenuItems(dropdownMenuEl.querySelectorAll('.menuitem'));
        expect(dropdown.menuItemLinks.length).to.equal(2);
    });

    describe('addMenuItemLink method:', function () {
        var menuItemLink = document.createElement('a'),
            dropdownWrapper = document.createElement('div');

        beforeEach(function() {
            dropdownMenuEl = getDropdownMenuDOM();
        });

        it('should add menu item link to dropdown menu component and hide from screen reader', function() {
            dropdown.addMenuItemLink(menuItemLink, dropdownWrapper);
            expect(dropdownWrapper.getAttribute('aria-hidden')).to.equal('true');
            expect(dropdown.menuItemLinks.length).to.equal(3);
            expect(dropdown.menuItemLinks[2]).to.be.equal(menuItemLink);
        });

        it('should not add one menu item link twice to dropdown menu component', function() {
            dropdown.addMenuItemLink(menuItemLink, dropdownWrapper);
            expect(dropdown.menuItemLinks.length).to.equal(3);
            expect(dropdown.menuItemLinks[2]).to.be.equal(menuItemLink);
        });

        it('should returns false if try to add menuitem to DropdownMenu component with one param', function() {
            expect(dropdown.addMenuItemLink(menuItemLink)).to.be.equal(false);
        });

        it('should returns false if try to add menu item to dropdown menu component without param', function() {
            expect(dropdown.addMenuItemLink()).to.be.equal(false);
        });
    });

    it('should toggle menuitem dropdown', function() {
        var dropdown = new DropdownMenu(dropdownMenuEl, options);

        dropdown.toggleDropdown(dropdown.menuItemLinks[0]);

        expect(dropdown.menuItemLinks[0].previousElementSibling.checked).to.be.equal(true);
        expect(dropdown.menuItemLinks[1].previousElementSibling.checked).to.be.equal(false);
    });

    it('should toggle the aria-hidden attribute of the dropdown and the tabindex of links inside', function() {
        var dropdown = new DropdownMenu(dropdownMenuEl, options);
        dropdown.toggleAccessibility(dropdown.menuItemLinks[0], dropdown.menuItemLinks[0]);

        var clickedMenu = dropdown.menuItemLinks[0].nextElementSibling;
        var otherMenu = dropdown.menuItemLinks[1].nextElementSibling;

        expect(clickedMenu.getAttribute('aria-hidden')).to.be.equal('false');
        expect(otherMenu.getAttribute('aria-hidden')).to.be.equal('true');
        expect(clickedMenu.querySelectorAll('a')[0].getAttribute('tabindex')).to.be.equal('0');
        expect(otherMenu.querySelectorAll('a')[0].getAttribute('tabindex')).to.be.equal('-1');
    });

    function getDropdownMenuDOM() {
        var domEl = document.createElement('div'),
            html = '<ul>'
                + '<li class="menuitem"><input type="checkbox" /><a href="#" class="menuitem-link">Link 1</a><div class="dropdown-wrapper"><a href="#" tabindex="-1">Link 1</a><a href="#" tabindex="-1">Link 2</a></div></li>'
                + '<li class="menuitem"><input type="checkbox" /><a href="#" class="menuitem-link"> Link 2</a><div class="dropdown-wrapper"><a href="#" tabindex="-1">Link 1</a><a href="#" tabindex="-1">Link 2</a></div></li>'
                + '</ul>';

        domEl.innerHTML = html;
        return domEl;
    }
});