import Header from '../js/Index';
import HeaderLinks from '../js/links/Index';
import util from '../js/util';
import Search from '../js/Search';
import HdrSiteStripe from '../js/HdrSiteStripe';
import SavedItems from '../js/savedItems/SavedItems';
import Breakpoint from '../js/navigation/Breakpoint';
import LoyaltyDropdown from '../js/loyalty/Index';
import MiniBagOverlay from '../js/miniBag/Index';
import cookieBanner from '../js/cookieBanner';

describe('Header entry point', () => {
    let header,
        sandbox,
        mockConfig = {
            storeId: '424',
            langId: '3434',
            catalogId: '4343',
            CMD: {
                MSSavedBasketDisplayCmd: 'MSBlah'
            },
            wcsBaseUrl: '/blah/balh'

        };

    beforeEach(() => {
        sandbox = sinon.sandbox.create();
        document.body.appendChild(getHeaderLinksDOM());
        sandbox.stub(HeaderLinks.prototype, 'show');
        sandbox.stub(Search.prototype, 'init');
        sandbox.stub(SavedItems.prototype, 'init');
        sandbox.stub(MiniBagOverlay, 'initialize');
        sandbox.stub(util, 'setTabIndex');
        sandbox.stub(HdrSiteStripe, 'initHdrSiteStripeCarousel');
        sandbox.stub(cookieBanner, 'bind');
        Breakpoint.size = sinon.stub().returns('large');
        header = new Header(mockConfig);
    });

    afterEach(() => {
        sandbox.restore();
    });

    describe('constructor Method', () => {
        it('construct a new header instance', () => {
            expect(header).to.be.instanceof(Header);
        });
    });

    describe('Header Links', () => {
        it('shows the header links via the header links object', () => {
            expect(HeaderLinks.prototype.show).to.have.been.calledOnce;
        });
    });

    describe('Search', () => {
        it('initializes the global search bar object', () => {
            expect(Search.prototype.init).to.have.been.calledOnce;
        });
    });
    
    describe('Sets tab order', () => {
        it('initialises the tab order', () => {
            expect(util.setTabIndex).to.have.been.calledOnce;
        });
    });

    describe('Mini bag overlay', () => {
        it('initializes the mini bag overlay object for medium sized devices', () => {
            MiniBagOverlay.initialize.reset();
            Breakpoint.size = sinon.stub().returns('medium');
            header = new Header(mockConfig);
            expect(MiniBagOverlay.initialize).to.have.been.calledOnce;
        });

        it('initializes the mini bag overlay object for large sized devices', () => {
            MiniBagOverlay.initialize.reset();
            Breakpoint.size = sinon.stub().returns('large');
            header = new Header(mockConfig);
            expect(MiniBagOverlay.initialize).to.have.been.calledOnce;
        });

        it('initializes the mini bag overlay object for iPad devices', () => {
            MiniBagOverlay.initialize.reset();
            Breakpoint.size = sinon.stub().returns('small');
            header = new Header(mockConfig);
            expect(MiniBagOverlay.initialize).to.have.been.called;
        });
    });

    function getHeaderLinksDOM() {
        let domEl = document.createElement('div'),
            html = `<header class="header">
                        <section id="progressiveHeaderSection">
                            <div class="navigation-dropdown__panel-wrapper navigation-dropdown__panel--search">
                                <div class="navigation-res-nav-header__dropdown--search navigation-nav-header__dropdown">
                                    <form name="productSearch" action="/MSFindItemsByKeyword" method="GET" role="search">
                                      <fieldset class="navigation-search-bar--fieldset">
                                        <input type="hidden" id="searchSuggestionCmd" value="MSAutoSuggest">
                                      </fieldset>
                                    </form>
                                </div>
                            </div>
                            <nav class="navigation-top-links" id="menuBarTop">
                                <ul>
                                    <li class="navigation-res-nav-header__item navigation-res-nav-header__item--country dropdown">
                                    </li>
                                    <li class="navigation-res-nav-header__item navigation-res-nav-header__item--location">
                                    </li>
                                    <li class="navigation-res-nav-header__item navigation-res-nav-header__item--menu">
                                    </li>
                                    <li class="navigation-res-nav-header__item help">
                                    </li>
                                    <li class="navigation-res-nav-header__item navigation-res-nav-header__item--login navigation-res-nav-header__item--hidden">
                                    </li>
                                    <li class="navigation-res-nav-header__item navigation-res-nav-header__item--register navigation-res-nav-header__item--hidden">
                                    </li>
                                    <li class="navigation-res-nav-header__item navigation-res-nav-header__item--myaccount navigation-myaccount dropdown">
                                    </li>
                                    <li class="navigation-res-nav-header__item navigation-res-nav-header__item--sparks dropdown" id="sparksDropdown">
                                        <a class="navigation-header__sparks-link navigation-dropdown__link" href="/loyalty/loyaltyhub" data-sparks-hub-link="/loyalty/loyaltyhub" data-sparks-login-link="/loyalty/signup" data-analytics-link-name="header:sparks click">
                                            Sparks
                                        </a>
                                        <div class="navigation-dropdown__panel-wrapper">
                                          <div class="navigation-header__sub-list-container navigation-header__sub-list-container--sparks navigation-sparks-dropdown">
                                            <div class="navigation-sparks-dropdown__content">
                                              <p class="navigation-sparks-dropdown__welcome">Hi <span class="navigation-sparks-dropdown__user">George</span></p>
                                              <p class="navigation-sparks-dropdown__sparks"><span class="navigation-sparks-dropdown__sparks-count">4078</span> sparks</p>
                                              <ul class="navigation-sparks-dropdown__active-offers" style="display: none;">

                                              </ul>
                                              <div class="navigation-sparks-dropdown__selectable navigation-sparks-dropdown__avl-offers">
                                                  <a href="/loyalty/loyaltyhub#/offers" data-analytics-link-name="select offer">
                                                      <div class="navigation-sparks-dropdown__selectable-offer navigation-sparks-dropdown__selectable-offer--double  ">
                                                          <span>+</span>
                                                          <span class="navigation-sparks-dropdown__notification">3</span>
                                                      </div>
                                                      <span class="navigation-sparks-dropdown__selectable-label">Select an offer</span>
                                                  </a>
                                              </div>
                                              <div class="navigation-sparks-dropdown__selectable navigation-sparks-dropdown__selectable--bonus navigation-sparks-dropdown__avl-bonus-offers">
                                                  <a href="/loyalty/loyaltyhub#/offers" data-analytics-link-name="claim offer:bonus-offer">
                                                      <div class="navigation-sparks-dropdown__selectable-offer">
                                                          <span class="icon navigation-icon--sparks-filled"></span>
                                                          <span class="navigation-sparks-dropdown__notification">2</span>
                                                      </div>
                                                      <span class="navigation-sparks-dropdown__selectable-label">Claim your bonus</span>
                                                  </a>
                                              </div>
                                          </div>
                                            <a id="gotoSparksLink" data-sparks-hub-link="/loyalty/loyaltyhub" data-sparks-login-link="/loyalty/signup" data-analytics-link-name="sparks home" class="btn navigation-btn--north-secondary" href="/loyalty/loyaltyhub">
                                              GO TO SPARKS
                                            </a>
                                          </div>
                                        </div>
                                    </li>
                                    <li class="navigation-res-nav-header__item navigation-res-nav-header__item--bag">
                                        <a href="/webapp/wcs/stores/servlet/OrderCalculate?calculationUsageIdentifier=MSBasketView_ShoppingCartURL&amp;calculationUsageId=-1&amp;updatePrices=1&amp;catalogId=&amp;errorViewName=AjaxOrderItemDisplayView&amp;orderId=.&amp;langId=-24&amp;storeId=10151&amp;doPrice=Y&amp;URL=AjaxOrderItemDisplayView" class="navigation-header__link">
                                            <i class="navigation-primary__menu-icon navigation-res-icon--empty-my-bag"></i>
                                             <p class="res-nav-header__title">
                                              <span">Your bag</span><span class="navigation-nav-header__bag-count">0</span>
                                            </p>
                                        </a>
                                    </li>
                                    <li class="navigation-res-nav-header__item av-header__item navigation-res-nav-header__item--checkout navigation-res-nav-header__item--hidden">
                                    </li>
                                </ul>
                            </nav>
                            <span class="navigation-header__welcome" data-welcome-text="Welcome"></span>
                        </section>
                    </header>`;

        domEl.innerHTML = html;
        return domEl;
    }
});
