define([
    '../js/MyBag',
    'text!../partials/default/responsiveHeader.hbs',
    '../js/links/cookieService',
    '../../../lib/domQ',
    '../js/miniBag/Index',
], function (MyBag, headerMarkup, mnsCookieService, domQ, minibagOverlay) {

    describe('MyBag', function () {

        var header = null,
            myBagHeader = null;

        beforeEach(function () {
            var divElement = document.createElement('div');

            divElement.innerHTML = headerMarkup;
            header = divElement.firstChild;

            sinon.stub(mnsCookieService, 'getBasketSizeCookieValue');

            myBagHeader = new MyBag({
                basketCount: mnsCookieService.getBasketSizeCookieValue.returns(4),
                myBagCount: header.querySelector('.navigation-nav-header__bag-count'),
                myBagElement: header.querySelector('.navigation-res-nav-header__item--bag'),
                checkoutElement: header.querySelector('.navigation-res-nav-header__item--checkout'),
                myBagIconElement: header.querySelector('.navigation-res-nav-header__item--bag .navigation-primary__menu-icon')

            });
        });

        afterEach(function () {
            mnsCookieService.getBasketSizeCookieValue.restore();
        });

        it('should set the bag count when initialised from the cookie', function () {
          expect(parseInt(myBagHeader.options.myBagCount.innerHTML)).to.equal(4);
        });        

        it('should style the bag as selected when initialised from the cookie', function () {
          expect(domQ(header.querySelector('.navigation-res-nav-header__item--bag')).hasClass('selected')).to.be.true;
        });        

        it('should show the checkout link when initialised from the cookie', function () {
          expect(domQ(header.querySelector('.navigation-res-nav-header__item--checkout')).hasClass('navigation-res-nav-header__item--hidden')).to.be.false;
        });        

        describe('Updating the bag', function() {

            var evt = null;
          
            beforeEach(function() {
              // set the bag count to zero
              evt = document.createEvent('Event');
              evt.initEvent('updateItemCount', true, true);
              evt.myBagCount = 0;
              document.dispatchEvent(evt);            
            });

            it('should style the basket as unselected when all items removed from basket and hide the checkout', function () {
                expect(parseInt(myBagHeader.options.myBagCount.innerHTML)).to.equal(0);
                expect(domQ(header.querySelector('.navigation-res-nav-header__item--bag')).hasClass('selected')).to.be.false;
                expect(domQ(header.querySelector('.navigation-res-nav-header__item--checkout')).hasClass('navigation-res-nav-header__item--hidden')).to.be.true;
            });

            it('should style the basket as selected when an item is added to the basket and show the checkout', function () {
                evt.myBagCount = 1;
                document.dispatchEvent(evt);

                expect(parseInt(myBagHeader.options.myBagCount.innerHTML)).to.equal(1);
                expect(domQ(header.querySelector('.navigation-res-nav-header__item--bag')).hasClass('selected')).to.be.true;
                expect(domQ(header.querySelector('.navigation-res-nav-header__item--checkout')).hasClass('navigation-res-nav-header__item--hidden')).to.be.false;
            });
        });
    });
});