import Search from '../js/Search';
import jsdom from 'jsdom';
import path from 'path';

var mockSearchData = ['jean style', 'jewellery boxes and trinkets', 'jewellery sets', 'jersey tops', 'boys trousers & jeans', 'jeans', 'girls leggings & jeggings', 'royal jelly', 'jersey top', 'jegging', 'jeans'],
    mockSearchDataSmall = ['jean style', '', 'jewellery boxes and trinkets'],
    mockSearchDataCategories = ['<a href="&categoryTypeAhead=Womens">jean style</a>', '<a href="&categoryTypeAhead=Womens">jewellery boxes and trinkets</a>', '<a href="">jewellery sets</a>', '<a href="">jersey tops</a>', '<a href="">boys trousers & jeans</a>', '<a href="">jeans</a>', '<a href="">girls leggings & jeggings</a>', '<a href="">royal jelly</a>', '<a href="">jersey top</a>', '<a href="">jegging</a>', '<a href="">jeans</a>'],
    document,
    search,
    searchContainer = null,
    searchCommand = 'searchCommand',
    contentWindow,
    searchHeader = null,
    navVeil = null,
    searchContainerParentOverlay = null,
    searchButton = null,
    config = {
        wcsBaseUrl: '/webapp/wcs/stores/servlet/'
    };

describe('Search Control Component', function () {

    beforeEach(function (done) {

        var localStorageMock = {
            getItem: function () {
            },
            setItem: function () {
            },
            removeItem: function () {
            }
        };

        jsdom.env({
            file: path.resolve(__dirname, '../partials/default/searchOverlay.hbs'),
            done: function (err, window) {
                contentWindow = window;
                document = contentWindow.document;
                searchContainer = document.querySelector('.navigation-res-nav-header__dropdown--search');
                navVeil = document.querySelector('.navigation-veil-search');
                searchContainerParentOverlay = document.querySelector('.navigation-search-overlay');
                searchButton = document.querySelector('.navigation-btn__icon--search');
                search = new Search({
                    'searchContainer': searchContainer,
                    'command': config.wcsBaseUrl + searchCommand,
                    'localStorage': localStorageMock,
                    'navVeil': navVeil,
                    'searchContainerParentOverlay': searchContainerParentOverlay,
                    'searchButton': searchButton
                });
                search.init();
                done();
            }
        });
    });

    afterEach(function () {
        contentWindow.close();
    });

    describe('constructor Method', function () {
        it('should return Search instance', function () {
            expect(search).to.be.instanceof(Search);
        });
    });

    describe('getSuggestionsUrl Method', function () {
        it('should return url with get params', function () {
            var stub = sinon.stub(search.utils, 'serialize').returns('param1=value1');
            search.command = searchCommand;
            expect(search.getSuggestionsUrl()).to.equal(searchCommand + '?param1=value1');
            stub.restore();
        });
    });

    describe('init Method', function () {
        it('should set default values to properties', function () {
            expect(search.searchContainer).to.equal(searchContainer);
            expect(search.command).to.equal(config.wcsBaseUrl + searchCommand);
            expect(search.submitButton.hasAttribute('disabled')).to.equal(true);
            expect(search.submitButton).to.equal(document.querySelector('.navigation-search-bar--submit'));
            expect(search.searchBar).to.equal(document.querySelector('.navigation-search-bar--search'));
            expect(typeof search.formParams).to.equal('object');
            expect(search.formParams.q).to.equal('');
            expect(search.wrapperGenerated).to.equal(true);
            expect(search.visualSearch).to.equal(document.querySelector('.navigation-visual-search'));
            expect(search.clearTextSearchOverlay).to.equal(document.querySelector('.navigation-res-nav-header__dropdown--search .navigation-hdr-search-bar--clear'));
            expect(search.closeSearchOverlayButton).to.equal(document.querySelector('.navigation-res-nav-header__dropdown--search .navigation-search-bar--clear'));
            expect(search.searchContainerParentOverlay).to.equal(document.querySelector('.navigation-search-overlay'));
        });

        it('should call bindEvents method', function () {
            var spy = sinon.spy(Search.prototype, 'bindEvents');

            search = new Search({
                'searchContainer': searchContainer,
                'command': config.wcsBaseUrl + searchCommand,
                'navVeil': navVeil,
                'searchContainerParentOverlay': searchContainerParentOverlay,
                'searchButton': searchButton
            });
            search.init();
            expect(spy.calledOnce).to.be.ok;
            Search.prototype.bindEvents.restore();
        });
    });

    describe('createSuggestions Method', function () {
        var recentSearchSuggestionsStub;

        beforeEach(function () {
            recentSearchSuggestionsStub = sinon.stub(search, 'addRecentSearchSuggestions', function () {
            });
        });

        afterEach(function () {
            recentSearchSuggestionsStub.restore();
        });

        it('should create max 10 suggestions when data is more than 10', function () {
            search.createSuggestions(mockSearchData);

            expect(search.SearchdropdownDiv.children[0].children.length).to.equal(10);
        });

        it('should remove any empty string and create suggestions equal to rest of the data length when data is less than 10', function () {
            search.createSuggestions(mockSearchDataSmall);

            expect(search.SearchdropdownDiv.children[0].children.length).to.equal(mockSearchDataSmall.length - 1);
        });

        it('should open suggestion list if input length is greater than 1', function () {
            search.createSuggestions(mockSearchDataSmall);

            expect(search.wrapper.className.indexOf('navigation-search-bar--suggestion-dropdown')).to.equal(0);
        });

        it('should flag the end of the category section by adding a class', function () {
            search.createSuggestions(mockSearchDataCategories);

            expect(search.searchContainer.getElementsByClassName('suggestion-item--category').length).to.equal(1);
        });
    });

    describe('enableSubmit Method', function () {
        it('should enable submit button if input length is greater than 1', function () {
            search.command = 'http://test';
            search.searchInputKeyUpHandler({
                shiftKey: false,
                target: {
                    value: 'Jeans'
                }
            });

            expect(search.submitButton.hasAttribute('disabled')).to.equal(false);
            expect(search.submitButton.className.indexOf('navigation-search-bar--submit__active')).to.not.equal(-1);
        });
    });

    describe('disableSubmit Method', function () {
        var recentSearchSuggestionsStub;

        beforeEach(function () {
            recentSearchSuggestionsStub = sinon.stub(search, 'addRecentSearchSuggestions', function () {
            });
        });

        afterEach(function () {
            recentSearchSuggestionsStub.restore();
        });

        it('should disable submit button if input length is less than 2', function () {
            search.searchInputKeyUpHandler({
                shiftKey: false,
                target: {
                    value: 'J'
                }
            });

            expect(search.submitButton.hasAttribute('disabled')).to.equal(true);
            expect(search.submitButton.className.indexOf('navigation-search-bar--submit__active')).to.equal(-1);
        });
    });

    describe('selectSuggestion Method', function () {
        var selectSuggestionSpy;
        var recentSearchSuggestionsStub;

        beforeEach(function () {
            selectSuggestionSpy = sinon.spy(Search.prototype, 'selectSuggestion');
            recentSearchSuggestionsStub = sinon.stub(search, 'addRecentSearchSuggestions', function () {
            });
        });

        afterEach(function () {
            Search.prototype.selectSuggestion.restore();
            recentSearchSuggestionsStub.restore();
        });

        it('should able to select suggestion from list', function () {
            var element;

            search.createSuggestions(mockSearchDataSmall);
            element = search.wrapper.querySelectorAll('.navigation-search-bar__suggestion-list li')[0];
            // trigger event
            element.click();
            expect(search.searchBar.value).to.equal(element.textContent);
            expect(selectSuggestionSpy.calledOnce).to.be.ok;
        });
    });

    describe('closeSuggestions Method', function () {
        var recentSearchSuggestionsStub;

        beforeEach(function () {
            recentSearchSuggestionsStub = sinon.stub(search, 'addRecentSearchSuggestions', function () {
            });
        });

        afterEach(function () {
            recentSearchSuggestionsStub.restore();
        });

        it('should close suggestion list if input length is less than 2', function (done) {
            search.createSuggestions(mockSearchDataSmall);
            search.closeSuggestions();
            expect(search.wrapper.className.indexOf('navigation-search-bar--suggestion-dropdown__hidden')).to.not.equal(-1);
            done();
        });
    });

    describe('generateWrapper Method', function () {
        it('should able to generate wrapper for suggestion list', function () {
            search.generateWrapper();

            expect(search.wrapperGenerated).to.equal(true);
        });
    });

    describe('createSuggestionItem Method', function () {
        it('should create suggestion item', function () {
            var item = 'men jeans',
                listItem;

            listItem = search.createSuggestionItem(item);

            expect(listItem.textContent).to.equal(item);
        });
    });

    describe('getRecentSearches Method', function () {
        it('should call localStorage.getItem', function () {
            var localStorageSpy = sinon.spy(search.localStorage, 'getItem');

            search.getRecentSearches();

            expect(localStorageSpy).calledWith('myRecentSearch');

            localStorageSpy.reset();
        });

        it('should return an array of search items when found in localStorage', function () {
            var localStorageStub = sinon.stub(search.localStorage, 'getItem', function () {
                return 'socks||shoes||dresses';
            });

            var recentSearches = search.getRecentSearches();

            expect(recentSearches).to.eql(['socks', 'shoes', 'dresses']);

            localStorageStub.reset();
        });

        it('should return an empty array when NOT found in localStorage', function () {
            var localStorageStub = sinon.stub(search.localStorage, 'getItem', function () {
                return null;
            });

            var recentSearches = search.getRecentSearches();

            expect(recentSearches).to.eql([]);

            localStorageStub.reset();
        });
    });

    describe('setRecentSearches Method', function () {
        var localStorageSpy, localStorageStub;

        beforeEach(function () {
            localStorageSpy = sinon.spy(search.localStorage, 'setItem');
            localStorageStub = sinon.stub(search.localStorage, 'getItem', function (item) {
                if (item === 'myRecentSearch') {
                    return 'socks||shoes||dresses';
                } else {
                    return null;
                }
            });
        });

        afterEach(function () {
            localStorageSpy.reset();
            localStorageStub.reset();
        });

        it('should call localStorage.setItem', function () {
            search.setRecentSearches('trousers');
            expect(localStorageSpy).calledWith('myRecentSearch', 'trousers||socks||shoes');
        });

        it('should not keep duplicate search term but put that term to the top of the list', function () {
            search.setRecentSearches('shoes');
            expect(localStorageSpy).calledWith('myRecentSearch', 'shoes||socks||dresses');
        });

    });

    describe('clearRecentSearches Method', function () {
        it('should call localStorage.removeItem', function () {
            var localStorageSpy = sinon.spy(search.localStorage, 'removeItem');

            search.clearRecentSearches();

            expect(localStorageSpy).calledWith('myRecentSearch');

            localStorageSpy.reset();
        });
    });

    describe('addRecentSearchSuggestions Method', function () {
        var getRecentSearchesStub;

        beforeEach(function () {
            getRecentSearchesStub = sinon.stub(search, 'getRecentSearches', function () {
                return ['shoes', 'socks', 'dresses'];
            });

            search.addRecentSearchSuggestions();
        });

        afterEach(function () {
            getRecentSearchesStub.restore();
        });

        it('should create a title element', function () {
            var titleElement = search.recentSearchDiv;
            expect(titleElement.innerHTML).to.equal('<b>Recent searches</b>');
        });

        it('should create the appropriate number of recent search elements', function () {
            var recentSearchElements = search.recentsearchul.querySelectorAll('.suggestion-item:not(.navigation-suggestion-item--clear)');
            expect(recentSearchElements.length).to.equal(3);
        });

        it('should add a clear element', function () {
            var clearElements = search.searchContainer.querySelector('.navigation-suggestion-item--clear-text');
            expect(clearElements.innerHTML).to.equal('Clear recent searches');
        });
    });

    describe('createRecentSearchLinks Method', function () {
        it('should create the correct number of a-link elements', function () {
            var recentSearchLinks = search.createRecentSearchLinks(['shoes', 'socks', 'dresses']);
            expect(recentSearchLinks.length).to.equal(3);
        });

        it('should add the correct data parameters', function () {
            var expectedLinkElement = '<a href="/MSFindItemsByKeyword?%7B%7B%20key%20%7D%7D=%7B%7B%20value%20%7D%7D&amp;searchTerm=shoes" class="reSearchHeader" data-keyword="shoes">shoes</a>';

            var recentSearchLinks = search.createRecentSearchLinks(['shoes']);
            var aLink = recentSearchLinks[0];

            expect(aLink).to.equal(expectedLinkElement);
        });
    });
});

