import HeaderLinks from '../../js/links/Index';
import mnsCookieService from '../../js/links/cookieService';
import helper from './helper/helper';
import analytics from '../../js/analytics';

describe('Progressive Header HeaderLinks component', function() {

    var headerLinksDOM;

    beforeEach(function() {
        headerLinksDOM = helper.getHeaderLinksDOM();
        document.body.appendChild(headerLinksDOM);
    });

    afterEach(function() {
        headerLinksDOM.remove();
    });

    describe('constructor method', function() {
        it('should have correct member variables', function() {
            var headerLinks = new HeaderLinks(headerLinksDOM);

            expect(headerLinks.nav).to.be.equal(headerLinksDOM.querySelector('nav'));
            expect(headerLinks.signIn).to.be.equal(headerLinksDOM.querySelector('.navigation-res-nav-header__item--login'));
            expect(headerLinks.signUp).to.be.equal(headerLinksDOM.querySelector('.navigation-res-nav-header__item--register'));
            expect(headerLinks.signInLink).to.be.equal(headerLinksDOM.querySelector('.navigation-res-nav-header__login-link'));
            expect(headerLinks.myAccount).to.be.equal(headerLinksDOM.querySelector('.navigation-res-nav-header__item--myaccount'));
            expect(headerLinks.welcomeMessage).to.be.equal(headerLinksDOM.querySelector('.navigation-header__welcome'));
            expect(headerLinks.tabletCheckout).to.be.equal(headerLinksDOM.querySelector('.navigation-res-nav-header__item--checkout'));
            expect(headerLinks.viewBagCount).to.be.equal(headerLinksDOM.querySelector('.navigation-nav-header__bag-count'));
        });
    });

    describe('show method', function() {

        var headerLinks;

        beforeEach(function() {
            headerLinks = new HeaderLinks(headerLinksDOM);
            sinon.stub(headerLinks, 'getUserSegmentSearchSwitch');
            sinon.stub(headerLinks, 'toggleLoginLink');
            sinon.stub(headerLinks, 'toggleCheckoutLink');
            sinon.stub(headerLinks, 'updateViewBagCount');
            sinon.stub(headerLinks, 'redirectSignInLink');
            sinon.stub(headerLinks, 'bindHeaderLinkEvents');
        });

        it('should show header links navigation after toggling links', function() {
            headerLinks.show();

            expect(headerLinks.toggleLoginLink.calledOnce).to.be.ok;
            expect(headerLinks.toggleCheckoutLink.calledOnce).to.be.ok;
            expect(headerLinks.updateViewBagCount.calledOnce).to.be.ok;
            expect(headerLinks.redirectSignInLink.calledOnce).to.be.ok;
            expect(headerLinks.bindHeaderLinkEvents.calledOnce).to.be.ok;
            expect(headerLinks.nav.className.indexOf('navigation-top-links--hidden')).be.equal(-1);
        });
    });

    describe('toggleLink method', function() {
        var stubIsUserLoggedIn,
            headerLinks;

        beforeEach(function() {
            headerLinks = new HeaderLinks(headerLinksDOM);
            stubIsUserLoggedIn = sinon.stub(mnsCookieService, 'isUserLoggedIn')
            sinon.stub(mnsCookieService, 'getUsername').returns('TestUser');
        });

        afterEach(function() {
            mnsCookieService.isUserLoggedIn.restore();
            mnsCookieService.getUsername.restore();
        });

        it('should show login and register links if MS_USER_COOKIE is not available', function() {
            stubIsUserLoggedIn.returns(false);
            headerLinks.toggleLoginLink();

            expect(headerLinks.signIn.className.indexOf('navigation-res-nav-header__item--hidden')).to.be.equal(-1);
            expect(headerLinks.myAccount.className.indexOf('navigation-res-nav-header__item--hidden')).to.be.above(-1);
        });

        it('should show my account link if MS_USER_COOKIE is available', function() {
            stubIsUserLoggedIn.returns(true);
            headerLinks.toggleLoginLink();

            expect(headerLinks.signIn.className.indexOf('navigation-res-nav-header__item--hidden')).to.be.above(-1);
            expect(headerLinks.myAccount.className.indexOf('navigation-res-nav-header__item--hidden')).to.be.equal(-1);
            expect(headerLinks.welcomeMessage.textContent).to.be.equal('TestUser');
        });
    });

    describe('updateViewBagCount method', function() {
        beforeEach(function() {
            sinon.stub(mnsCookieService, 'getBasketSizeCookieValue').returns(4);
        });

        afterEach(function() {
            mnsCookieService.getBasketSizeCookieValue.restore();
        });

        it('should update view bag count element text', function() {
            var headerLinks = new HeaderLinks(headerLinksDOM);

            headerLinks.updateViewBagCount(4);
            expect(headerLinks.viewBagCount.textContent).to.be.equal('4');
        });
    });
});
