import OffersParser from '../../js/loyalty/OffersParser';
import Helper from './helper/helper';

let offerBreakdown,
    data;

describe('Progressive Header Loyalty offers parser', function() {

    beforeEach(function() {
        offerBreakdown = Helper.getOfferBreakdown();
        data = OffersParser.parse(offerBreakdown);
    });

    it('should parse customer sparks from offer breakdown', function() {
        expect(data.customerSparks).to.equal(offerBreakdown.sparks);
    });

    it('should parse total offers from offer breakdown', function() {
        expect(data.totalOffers).to.equal(offerBreakdown.numberOfOffers);
    });

    it('should parse slotted offers from offer breakdown', function() {
        expect(data.slottedOffers).to.equal(offerBreakdown.slotted.slots);
    });

    it('should parse unlocked offers from offer breakdown', function() {
        let unlockedOffers = offerBreakdown.slotted.slots.filter(function (offerSlot) {
            return offerSlot.empty && offerSlot.unlocked;
        });

        expect(data.unlockedOffers).to.deep.equal(unlockedOffers);
    });

    it('should parse all bonus offers from offer breakdown', function() {
        let allBonusOffers = offerBreakdown.allBonusOffers.BONUS.slots
            .concat(offerBreakdown.allBonusOffers.FRIENDS_AND_FAMILY.slots)
            .concat(offerBreakdown.allBonusOffers.FRIENDS_AND_FAMILY_STAFF.slots);

        expect(data.allBonusOffers).to.deep.equal(allBonusOffers);
    });

    it('should parse all offers from offer breakdown', function() {
        let allOffers = offerBreakdown.slotted.slots
            .concat(offerBreakdown.allBonusOffers.BONUS.slots)
            .concat(offerBreakdown.allBonusOffers.FRIENDS_AND_FAMILY.slots)
            .concat(offerBreakdown.allBonusOffers.FRIENDS_AND_FAMILY_STAFF.slots);

        expect(data.allOffers).to.deep.equal(allOffers);
    });

    it('should parse all active offers from offer breakdown', function() {
        let allActiveOffers = offerBreakdown.slotted.slots
            .concat(offerBreakdown.allBonusOffers.BONUS.slots)
            .concat(offerBreakdown.allBonusOffers.FRIENDS_AND_FAMILY.slots)
            .concat(offerBreakdown.allBonusOffers.FRIENDS_AND_FAMILY_STAFF.slots)
            .filter(function (offerSlot) {
                return !offerSlot.available && offerSlot.offer;
            });

        expect(data.allActiveOffers).to.deep.equal(allActiveOffers);
    });

    it('should parse all available slotted offers from offer breakdown', function() {
        let availableSlottedOffers = offerBreakdown.slotted.slots
            .filter(function (offerSlot) {
                return offerSlot.available && offerSlot.free;
            });

        expect(data.availableSlottedOffers).to.deep.equal(availableSlottedOffers);
    });

    it('should parse all available bonus offers from offer breakdown', function() {
        let availableBonusOffers = offerBreakdown.allBonusOffers.BONUS.slots
            .concat(offerBreakdown.allBonusOffers.FRIENDS_AND_FAMILY.slots)
            .concat(offerBreakdown.allBonusOffers.FRIENDS_AND_FAMILY_STAFF.slots)
            .filter(function (offerSlot) {
                return offerSlot.available && offerSlot.free;
            });

        expect(data.availableBonusOffers).to.deep.equal(availableBonusOffers);
    });
});
