import DesktopDevice from '../../js/navigation/DesktopDevice';

describe('Desktop device', () => {

    let mockedLevel2 = {},
        mockedLevel3 = {},
        sandbox,
        desktopDevice;

    beforeEach(() => {
        sandbox = sinon.sandbox.create();
        mockedLevel2 = {
            activate: sandbox.spy(),
            deactivate: sandbox.spy(),
            navPrimaryAList: []
        };
        mockedLevel3 = {
            navPrimarySubmenuList: []
        };
        desktopDevice = new DesktopDevice(mockedLevel2, mockedLevel3);
    });

    afterEach(() => {
        sandbox.restore();
    });

    describe('constructor method', () => {

        it('adds a level2 onto the desk top device instance', () => {
            expect(desktopDevice.level2).to.equal(mockedLevel2);
        });

        it('adds a level3 onto the desk top device instance', () => {
            expect(desktopDevice.level3).to.equal(mockedLevel3);
        });

        it('defaults the menu over flag to false on the instantiated desktop device object', () => {
            expect(desktopDevice.menuOverFlag).to.equal(false);
        });

        it('defaults the panel over flag to false on the instantiated desktop device object', () => {
            expect(desktopDevice.panelOverFlag).to.equal(false);
        });
    });

    describe('bind function', () => {
        it('defines a function for binding desktop device events', () => {
            expect(typeof desktopDevice.bind).to.equal('function');
        });
    });

    describe('unbind function', () => {
        beforeEach(() => {
            desktopDevice.unbind();
        });

        it('defines a function for binding desktop device events', () => {
            expect(desktopDevice.timeoutId).to.be.undefined;
        });
    });

    describe('menu over function', () => {
        let mockedMouseEvent = {
            target: {
                someKey: 'someValue'
            }
        };

        beforeEach(() => {
            desktopDevice.menuOver(mockedMouseEvent);
        });

        it('sets the menu over flag to true', () => {
            expect(desktopDevice.menuOverFlag).to.equal(true);
        });

        it('sets the activation timeout function', () => {
            expect(desktopDevice.timeoutId).not.to.be.undefined;
            it('deactivates the level 2 navigation passing the element which called the event', () => {
                expect(desktopDevice.activate).to.have.been.calledwith(mockedMouseEvent.target);
            });
        });
    });

    describe('menu out function', () => {
        beforeEach(() => {
            sandbox.stub(desktopDevice, 'deactivate');
            desktopDevice.menuOut();
        });

        it('sets the menu over flag to false', () => {
            expect(desktopDevice.menuOverFlag).to.equal(false);
        });

        it('deactivates the level 2 navigation passing the element which called the event', () => {
            expect(desktopDevice.deactivate).to.have.been.calledOnce;
        });
    });

    describe('panel over function', () => {
        beforeEach(() => {
            desktopDevice.panelOver();
        });

        it('sets the panel over flag to true', () => {
            expect(desktopDevice.panelOverFlag).to.equal(true);
        });
    });

    describe('panel out function', () => {
        beforeEach(() => {
            sandbox.stub(desktopDevice, 'deactivate');
            desktopDevice.panelOut();
        });

        it('sets the panel over flag to false', () => {
            expect(desktopDevice.panelOverFlag).to.equal(false);
        });

        it('deactivates the desktop device instance', () => {
            expect(desktopDevice.deactivate).to.have.been.calledOnce;
        });
    });

    describe('deactivate function', () => {
        beforeEach(() => {
            desktopDevice.deactivate();
        });

        it('sets the deactivation timeout function', () => {
            expect(desktopDevice.timeoutId).not.to.be.undefined;
        });
    });

});