import Level2 from '../../js/navigation/Level2';
import Level3 from '../../js/navigation/Level3';
import XSmallDevice from '../../js/navigation/XSmallDevice';
import DesktopDevice from '../../js/navigation/DesktopDevice';
import LazyLoad from '../../js/navigation/LazyLoad';
var xhrPromise = require('xhr-promise');
describe('navigation level 2', () => {
    let mockedConfig = { someKey: 'someVal' }, sandbox;
    beforeEach(() => {
        sandbox = sinon.sandbox.create();
    });
    afterEach(() => {
        sandbox.restore();
    });
    describe('constructor method', () => {
        let level2;
        beforeEach(() => {
            sandbox.stub(Level2.prototype, 'init');
            level2 = new Level2(mockedConfig);
        });
        it('adds a config object onto the level 2 instance', () => {
            expect(level2.config).to.equal(mockedConfig);
        });
        it('calls the init method when instantiated', () => {
            expect(Level2.prototype.init).to.have.been.calledOnce;
        });
        it('add a lazy load instance onto the level 2 object ', () => {
            expect(level2.lazyload instanceof LazyLoad).to.be.true;
        });
    });
    describe('init method', () => {
        let level2, loadStub, xhrp;
        beforeEach(() => {
            loadStub = sandbox.stub(Level3.prototype, 'load');
            sandbox.stub(Level2.prototype, 'bindForDevice');
            xhrp = new xhrPromise();
            loadStub.returns(xhrp.send({ url: 'http://test' }));
            level2 = new Level2(mockedConfig);
            sandbox.stub(level2, 'bindForDevice');
            Level3.prototype.load.reset();
            document.body.appendChild(createwindowObject());
        });
        it('attempts to load the level 3 navigation', () => {
            level2.init();
            expect(Level3.prototype.load).to.have.been.calledOnce;

        });
        it('makes a call to bind the device event listeners when load of level3 navigation is successful', () => {
            level2.load();
            expect(level2.bindForDevice).to.have.been.calledOnce;
        });

        it('does NOT make a call to bind the device event listeners when load of level3 navigation is successful', () => {
            expect(level2.bindForDevice).not.to.have.been.called;
        });

        function createwindowObject() {
            let domEl = document.createElement('div'),
                html = `<script>
                window.MS_COMMON_L23CMDCALL=true;
                      </script>`;
            domEl.innerHTML = html;
            return domEl;
        }
    });
    describe('checkClickToClose method', () => {
        let level2;
        beforeEach(() => {
            sandbox.stub(Level2.prototype, 'init');
            sandbox.stub(Level2.prototype, 'deactivate');
            level2 = new Level2(mockedConfig);
            document.body.appendChild(createL2DOM());
        });
        it('does NOT deactivate the level 2 navigation when touching level 2 menu link ', () => {
            let mockedElement = document.querySelector('.nav-primary__menu-link'),
                mockedEvent = {
                    target: mockedElement
                };

            level2.checkClickToClose(mockedEvent);
            expect(level2.deactivate).not.to.have.been.called;
        });

        it('does deactivate the level 2 navigation when touch event NOT from a level 2 menu link ', () => {
            let mockedElement = document.createElement('div'),
                mockedEvent = {
                    target: mockedElement
                };

            level2.checkClickToClose(mockedEvent);
            expect(level2.deactivate).to.have.been.called;
        });

        function createL2DOM() {
            let domEl = document.createElement('div'),
                html = `<script>
                window.MS_COMMON_L23CMDCALL=true;
                      </script>
        <div id="navContainer">
                        <div class="nav-primary__menu-link"></div>
                        <div class="navigation-is-active"></div>
                    </div>`;

            domEl.innerHTML = html;
            return domEl;
        }

    });

});
